@@ -2,7 +2,7 @@
 /* file alloc.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2002 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 
@@ -34,12 +34,13 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** memory allocation and garbage collection functions
 **
 ** date of last change
-** 95-03-29
-** 96-10-04	changed iniconfig
-** 97-04-18	various SIZE defines, changed mvsymtab, tempfree
-** 97-09-06	memory allocation #ifdef M_LARGE
-** 97-12-26	small changes in iniconfig
-** 98-01-06	fixed small bug in moveobj
+** 1995-03-29
+** 1996-10-04	changed iniconfig
+** 1997-04-18	various SIZE defines, changed mvsymtab, tempfree
+** 1997-09-06	memory allocation #ifdef M_LARGE
+** 1997-12-26	small changes in iniconfig
+** 1998-01-06	fixed small bug in moveobj
+** 2002-04-05   changed some configuration constants
 */
 
 #include "common.h"
@@ -70,16 +71,16 @@ Email	forster@rz.mathematik.uni-muenchen.de
 #ifdef M_LARGE
 #include <assert.h>
 
-#define HASHTABSIZE	1009	/* size of hash table (prime) */
-#define BLOCKMAX	  64
+#define HASHTABSIZE	    1009	/* size of hash table (prime) */
+#define BLOCKMAX	     192    /* must be < 255 */
 #define BLOCK0SIZE     BIG_BLOCK0SIZE
 #define BLOCKSIZE      BIG_BLOCKSIZE
 #define RESERVE	       16000
-#define WORKSTKSIZE    16000
+#define WORKSTKSIZE    BIG_BLOCKSIZE
 #define ARGSTKSIZE     16000
 
 #ifdef MEM
-#if (MEM >= 1) && (MEM <= 16)
+#if (MEM >= 1) && (MEM <= 32)
 #define MEM_DEFAULT    (MEM*1024)
 #endif
 #endif /* MEM */
@@ -158,8 +159,8 @@ typedef struct {
 PRIVATE char *Stringpool;
 PRIVATE char *Stringsys;
 
-PRIVATE size_t symbot, userbot;
-PRIVATE size_t membot, memceil;
+PRIVATE size_t symBot, userBot;
+PRIVATE size_t memBot, memCeil;
 PRIVATE size_t argstkSize, workstkSize, blockSize, block0Size;
 PRIVATE int curblock, noofblocks, auxindex0, maxblocks;
 PRIVATE blkdesc blockinfo[BLOCKMAX+1];
@@ -198,8 +199,8 @@ PRIVATE int datupdate	_((int flg));
 /*------------------------------------------------------------*/
 PUBLIC void inialloc()
 {
-	gcsym	= newsymsig("gc",	sFBINARY, (truc)Fgc, s_01);
-	memavsym= newsymsig("memavail", sFBINARY, (truc)Fmemavail, s_01);
+	gcsym	= newsymsig("gc",	sFBINARY, (wtruc)Fgc, s_01);
+	memavsym= newsymsig("memavail", sFBINARY, (wtruc)Fmemavail, s_01);
 }
 /*--------------------------------------------------------------*/
 PRIVATE void iniconfig(mem)
@@ -209,7 +210,8 @@ int mem;
 	long memmax;
 
 #ifdef M_LARGE
-	assert(sizeof(int) > 2);
+	assert(sizeof(word4) == 4);
+	assert(sizeof(word2) == 2);
 #endif
 	argstkSize = ARGSTKSIZE;
 	workstkSize = WORKSTKSIZE;
@@ -359,7 +361,7 @@ int mem;
 	if(noofblocks < 2)
 		goto errmem;
 	Symbol = Memory[0];
-	symbot = 0;
+	symBot = 0;
 	Stringpool = (char *)(Symbol + block0Size);
 	iniblock();
 	return((int)(memallsize >> 10));
@@ -444,8 +446,8 @@ PRIVATE void iniblock()
 		blockinfo[m].curbot = blockSize/2;
 	}
 	curblock = 1;
-	membot = blockinfo[curblock].blkbot;
-	memceil = blockinfo[curblock].blkceil;
+	memBot = blockinfo[curblock].blkbot;
+	memCeil = blockinfo[curblock].blkceil;
 	AriScratch = (word2 *)Memory[noofblocks];
 
 #ifdef M_LARGE
@@ -458,18 +460,18 @@ PRIVATE void iniblock()
 /*---------------------------------------------------------*/
 /*
 ** must be called at the end of initializations
-** sets global variables userbot and Stringsys
+** sets global variables userBot and Stringsys
 ** returns number of bytes used by system for symbols and symbol names
 */
 PUBLIC int initend()
 {
 	size_t n;
 
-	userbot = symbot;
+	userBot = symBot;
 	Stringsys = Stringpool;
 
 	n = (char *)(Symbol + block0Size) - Stringsys;
-	n += sizeof(truc)*userbot;
+	n += sizeof(truc)*userBot;
 	return(n);
 }
 /*---------------------------------------------------------*/
@@ -575,7 +577,7 @@ truc obj, pack;
 	int sys;
 
 	v.xx = obj;
-	sys = (v.pp.ww < userbot);
+	sys = (v.pp.ww < userBot);
 	if(pack == arisym)
 		return(sys);
 	else if(pack == usersym)
@@ -601,7 +603,7 @@ long slot[4];
 		else {
 		    nact += c - b;
 		    if(flg < FULL) {
-			b = (i == curblock ? membot : blockinfo[i].curbot);
+			b = (i == curblock ? memBot : blockinfo[i].curbot);
 			nfree += c - b;
 		    }
 		}
@@ -609,7 +611,7 @@ long slot[4];
 	slot[0] = s * nres;
 	slot[1] = s * nact;
 	slot[2] = s * nfree;
-	nsymb = Stringpool - (char *)(Symbol + symbot);
+	nsymb = Stringpool - (char *)(Symbol + symBot);
 	slot[3] = nsymb;
 }
 /*---------------------------------------------------------*/
@@ -620,18 +622,19 @@ long s[];
 	long diff;
 
 	diff = s[1] - s[2];
-	n = s2form(OutBuf,"~8D Bytes reserved; ~D Bytes active ",s[0],s[1]);
-	s2form(OutBuf+n,"(~D used, ~D free)",diff,s[2]);
+	n = s2form(OutBuf,"~8D Bytes reserved; ~D Bytes active ",
+        (wtruc)s[0],(wtruc)s[1]);
+	s2form(OutBuf+n,"(~D used, ~D free)",(wtruc)diff,(wtruc)s[2]);
 	fprintline(tstdout,OutBuf);
 	s1form(OutBuf,
-	  "~8D Bytes free for user defined symbols and symbol names", s[3]);
+	  "~8D Bytes free for user defined symbols and symbol names", (wtruc)s[3]);
 	fprintline(tstdout,OutBuf);
 }
 /*---------------------------------------------------------*/
 PRIVATE void gcstatistics()
 {
 	fnewline(tstdout);
-	s1form(OutBuf,"total number of garbage collections: ~D",gccount);
+	s1form(OutBuf,"total number of garbage collections: ~D", (wtruc)gccount);
 	fprintline(tstdout,OutBuf);
 }
 /*---------------------------------------------------------*/
@@ -656,7 +659,7 @@ int argn;
 PUBLIC char *stringalloc(size)
 unsigned int size;	/* unit for size is sizeof(char) */
 {
-	if(Stringpool - size <= (char *)(Symbol + symbot))
+	if(Stringpool - size <= (char *)(Symbol + symBot))
 		faterr(err_memory);
 	Stringpool -= size;
 	return(Stringpool);
@@ -672,9 +675,13 @@ unsigned int size;	/* unit for size is sizeof(truc) */
 {
 	size_t loc;
 
-	loc = symbot;
-	symbot += size;
-	if(Stringpool <= (char *)(Symbol + symbot))
+	loc = symBot;
+	symBot += size;
+#ifdef ALIGN8
+    if (symBot & 0x1)
+        symBot++;
+#endif
+	if(Stringpool <= (char *)(Symbol + symBot))
 		faterr(err_memory);
 	return(loc);
 }
@@ -686,15 +693,15 @@ trucptr *ptraddr;
 {
 	variant v;
 
-	if(size > memceil - membot) {
+	if(size > memCeil - memBot) {
 		nextblock(size);
 	}
 	v.pp.b0 = flg;
 	v.pp.b1 = curblock;
-	v.pp.ww = membot;
-	*ptraddr = Memory[curblock] + membot;
+	v.pp.ww = memBot;
+	*ptraddr = Memory[curblock] + memBot;
 
-	membot += size;
+	memBot += size;
 	return(v.xx);
 }
 /*---------------------------------------------------------*/
@@ -724,9 +731,9 @@ unsigned int size;
 	int i,k;
 	int collected = 0;
 
-	blockinfo[curblock].curbot = membot;
+	blockinfo[curblock].curbot = memBot;
 	blockinfo[curblock].flag =
-		(memceil - membot >= NOAGERL ? HALFFULL : FULL);
+		(memCeil - memBot >= NOAGERL ? HALFFULL : FULL);
 
 	if(size > blockSize) {
 		reset(err_2large);
@@ -738,8 +745,8 @@ unsigned int size;
 			k = 1;
 		if((blockinfo[k].flag <= HALFFULL) &&
 		   (size <= blockinfo[k].blkceil - blockinfo[k].curbot)) {
-			membot = blockinfo[k].curbot;
-			memceil = blockinfo[k].blkceil;
+			memBot = blockinfo[k].curbot;
+			memCeil = blockinfo[k].blkceil;
 			curblock = k;
 			return;
 		}
@@ -801,8 +808,8 @@ PRIVATE void prepgc()
 			blockinfo[i].flag = FREE;
 			if(first) {
 				curblock = i;
-				membot = blockinfo[i].curbot;
-				memceil = blockinfo[i].blkceil;
+				memBot = blockinfo[i].curbot;
+				memCeil = blockinfo[i].blkceil;
 				first = 0;
 			}
 		}
@@ -815,7 +822,7 @@ PRIVATE void endgc()
 {
 	int scratchind, auxind;
 
-	blockinfo[curblock].curbot = membot;
+	blockinfo[curblock].curbot = memBot;
 
 	if(blockinfo[1].flag == RESERVED) {
 		scratchind = 1;
@@ -947,6 +954,7 @@ truc *ptr;
 
 	switch(type) {
 	case fBIGNUM:
+	case fGF2NINT:
 		len = ((struct bigcell *)ptr)->len;
 		return(SIZEOFBIG(len));
 	case fSTRING:
