@@ -83,6 +83,7 @@ PRIVATE int atan0	_((int prec, word2 *x, int n, word2 *z, word2 *hilf));
 PRIVATE int curfltprec;
 
 PRIVATE word2 LOG2DAT[]	 =	/* log(2) */
+#ifdef FPREC_HIGH
 {322, 0x154C, 0xB783, 0x64F1, 0xEC3F, 0x53DA, 0x501E, 0xF281, 0x9316, 
 0xDB4A, 0xF949, 0x56C9, 0xC921, 0xBE2E, 0x341C, 0x94F0, 0xF58D, 0xCA8, 0x4A00, 
 0xD287, 0x3D7, 0x554B, 0xE00C, 0x5497, 0x75DF, 0xFB0C, 0x2D06, 0xECA4, 0x850, 
@@ -107,8 +108,12 @@ PRIVATE word2 LOG2DAT[]	 =	/* log(2) */
 0x955D, 0xD2C, 0x1735, 0xA54, 0xC48, 0x7AA3, 0x5CFE, 0xB601, 0x74D, 0x8E82, 
 0x5E14, 0x7F8A, 0x6A9C, 0xA337, 0x3564, 0x9B33, 0x2566, 0x95D, 0xD1D6, 0x1E0B, 
 0x4C1A, 0x514C, 0x9393, 0x4E65, 0xCCCC, 0xCD33, 0xB479, 0xE732, 0xC943, 0x90E5,
-0xDB89, 0x775, 0x1746, 0xB396, 0x1400, 0x23DE, 0x7D2E, 0xFA15, 0xFC1E, 0x9D6D, 
-0xEE56, 0x51A2, 0x8FE5, 0x30F8, 0x610D, 0xFB90, 0xFB5B, 0xCA11, 0x7F4, 0xD5C6, 
+0xDB89, 0x775, 0x1746, 0xB396, 0x1400, 0x23DE, 0x7D2E, 0xFA15, 0xFC1E, 0x9D6D,
+0xEE56, 0x51A2, 0x8FE5, 0x30F8, 0x610D, 0xFB90, 0xFB5B, 0xCA11, 
+#else
+{66,
+#endif
+0x7F4, 0xD5C6, 
 0xF3F, 0x97C5, 0xDA2D, 0xE3A2, 0x2F20, 0xA187, 0x655F, 0x3248, 0x3830, 0xA6BD, 
 0xF5DF, 0x48CA, 0x9D65, 0x87B1, 0x72CE, 0xF74B, 0x7657, 0xA0EC, 0x256F, 0x603B,
 0xB136, 0x9BC3, 0xB9EA, 0x387E, 0x317C, 0xDA11, 0x1ACB, 0xE8C5, 0x224A, 0xCA16,
@@ -118,6 +123,7 @@ PRIVATE word2 LOG2DAT[]	 =	/* log(2) */
 0xD1CF, 0x17F7, 0xB172};
 
 PRIVATE word2 PI4THDAT[] =	 /* pi/4 */
+#ifdef FPREC_HIGH
 {322, 0x23A9, 0xE8F3, 0xBEC7, 0xC97F, 0x59E7, 0x1C9E, 0x900B, 0x4031, 
 0xB5A8, 0xC82, 0x4698, 0x702F, 0xD55E, 0xFEF6, 0x6E74, 0xD7CE, 0xF482, 0x1D03, 
 0xD172, 0xEA15, 0xF032, 0x92EC, 0xC64B, 0xCA01, 0x5983, 0xD2BF, 0x378C, 0xF401,
@@ -143,7 +149,11 @@ PRIVATE word2 PI4THDAT[] =	 /* pi/4 */
 0x6F4C, 0x5DF0, 0xB5C5, 0xA28F, 0xEC07, 0x83A2, 0x9B27, 0x8603, 0x180E, 0x772C,
 0xE39E, 0xCE3B, 0x2E36, 0x5E46, 0x3290, 0x217C, 0xCA18, 0x6C08, 0xF174, 0x9804,
 0x4ABC, 0x354E, 0x670C, 0x966D, 0x7096, 0x2907, 0x9ED5, 0x52BB, 0x2085, 0xF356,
-0x1C62, 0xAD96, 0xDCA3, 0x5D23, 0x8365, 0xCF5F, 0xFD24, 0x3FA8, 0x6916, 0xD39A,
+0x1C62, 0xAD96, 0xDCA3, 0x5D23, 0x8365, 0xCF5F, 0xFD24, 0x3FA8,
+#else
+{66,
+#endif
+0x6916, 0xD39A,
 0x1C55, 0x4836, 0x98DA, 0xBF05, 0xA163, 0x7CB8, 0xC200, 0x5B3D, 0xECE4, 0x6651,
 0x4928, 0x1FE6, 0x7C4B, 0x2411, 0xAE9F, 0x9FA5, 0x5A89, 0x6BFB, 0xEE38, 0xB7ED,
 0xF406, 0x5CB6, 0xBFF, 0xED6B, 0xA637, 0x42E9, 0xF44C, 0x7EC6, 0x625E, 0xB576, 
@@ -153,6 +163,7 @@ PRIVATE word2 PI4THDAT[] =	 /* pi/4 */
 0x2168, 0xDAA2, 0xC90F};
 
 PRIVATE word2 ATAN4DAT[] =	 /* atan(1/4) */
+#ifdef FPREC_HIGH
 {322, 0x6B3F, 0x5870, 0xADCF, 0xE549, 0x1C7F, 0x82B2, 0x9F6F, 0x5450, 
 0xE8F, 0xB2E1, 0x95F5, 0x9CE5, 0x998F, 0x64AE, 0x9F2F, 0xDE06, 0xF67, 0xF39, 
 0x111B, 0x3858, 0x25DF, 0x7580, 0x7910, 0xC3A6, 0x5FC3, 0xD1A8, 0xD87A, 0xE0C5,
@@ -178,7 +189,11 @@ PRIVATE word2 ATAN4DAT[] =	 /* atan(1/4) */
 0xEEBA, 0xE60, 0xDA37, 0x9A95, 0x1906, 0x77BF, 0xE0DF, 0xB528, 0xFCFC, 0xA1D, 
 0xDBFB, 0x2714, 0xFB05, 0x8AA1, 0x6928, 0xB682, 0x65A9, 0x50C7, 0x38B1, 0x4DD3,
 0x1BED, 0x4652, 0xD4F7, 0xE21E, 0xA8AB, 0xE37, 0x8EC9, 0xAEA6, 0x1541, 0x16BE, 
-0x3E03, 0xA59D, 0x5E8, 0x5574, 0x77B4, 0xA43E, 0x3638, 0x2903, 0x6F3D, 0xC10E, 
+0x3E03, 0xA59D, 0x5E8, 0x5574, 0x77B4, 0xA43E, 0x3638, 0x2903, 
+#else
+{66,
+#endif
+0x6F3D, 0xC10E, 
 0xF52C, 0x8AB6, 0x7616, 0xFD3E, 0x172D, 0x2301, 0x3520, 0x463F, 0x9382, 0xA65, 
 0xC31B, 0x1204, 0xD9EA, 0x1DE7, 0x6EE5, 0xF2CB, 0xCF96, 0x738F, 0xF848, 0x9C81,
 0xEB38, 0x48D5, 0x234, 0xF724, 0x5756, 0x7231, 0xFDA2, 0xA695, 0xF8FC, 0xADD0, 
@@ -194,16 +209,16 @@ PRIVATE word2 ATAN4DAT[] =	 /* atan(1/4) */
 /*----------------------------------------------------------------*/
 PUBLIC void inianalys()
 {
-	expsym	  = newsymsig("exp",   sFBINARY, (truc)Fexp, s_rr);
-	sqrtsym	  = newsymsig("sqrt",  sFBINARY, (truc)Fsqrt,s_rr);
-	sinsym	  = newsymsig("sin",   sFBINARY, (truc)Fsin, s_rr);
-	cossym	  = newsymsig("cos",   sFBINARY, (truc)Fcos, s_rr);
-	tansym	  = newsymsig("tan",   sFBINARY, (truc)Ftan, s_rr);
-	logsym	  = newsymsig("log",   sFBINARY, (truc)Flog, s_rr);
-	atansym	  = newsymsig("arctan",sFBINARY, (truc)Fatan,s_rr);
-	atan2sym  = newsymsig("arctan2",sFBINARY, (truc)Fatan2,s_rrr);
-	asinsym	  = newsymsig("arcsin",sFBINARY, (truc)Fasin,s_rr);
-	acossym	  = newsymsig("arccos",sFBINARY, (truc)Facos,s_rr);
+	expsym	  = newsymsig("exp",   sFBINARY, (wtruc)Fexp, s_rr);
+	sqrtsym	  = newsymsig("sqrt",  sFBINARY, (wtruc)Fsqrt,s_rr);
+	sinsym	  = newsymsig("sin",   sFBINARY, (wtruc)Fsin, s_rr);
+	cossym	  = newsymsig("cos",   sFBINARY, (wtruc)Fcos, s_rr);
+	tansym	  = newsymsig("tan",   sFBINARY, (wtruc)Ftan, s_rr);
+	logsym	  = newsymsig("log",   sFBINARY, (wtruc)Flog, s_rr);
+	atansym	  = newsymsig("arctan",sFBINARY, (wtruc)Fatan,s_rr);
+	atan2sym  = newsymsig("arctan2",sFBINARY, (wtruc)Fatan2,s_rrr);
+	asinsym	  = newsymsig("arcsin",sFBINARY, (wtruc)Fasin,s_rr);
+	acossym	  = newsymsig("arccos",sFBINARY, (wtruc)Facos,s_rr);
 
 	pisym	  = newsym("pi",   sSCONSTANT, inipi(FltPrec[MaxFltLevel]));
 }
