@@ -35,6 +35,8 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** date of last change
 ** 1994-12-31
 ** 2000-12-30: multiprec floats
+** 2002-02-16: changesign
+** 2002-04-21: chkintnz
 */
 
 #include "common.h"
@@ -43,12 +45,12 @@ PUBLIC int setfltprec	_((int prec));
 PUBLIC int deffltprec	_((void));
 PUBLIC int maxfltprec	_((void));
 PUBLIC int fltpreccode	_((int prec));
-PUBLIC int fltprec	_((int type));
+PUBLIC int fltprec	    _((int type));
 PUBLIC int refnumtrunc	_((int prec, truc *ptr, numdata *nptr));
 PUBLIC int getnumtrunc	_((int prec, truc *ptr, numdata *nptr));
 PUBLIC int getnumalign	_((int prec, truc *ptr, numdata *nptr));
 PUBLIC int alignfloat	_((int prec, numdata *nptr));
-PUBLIC int alignfix	_((int prec, numdata *nptr));
+PUBLIC int alignfix	    _((int prec, numdata *nptr));
 PUBLIC void adjustoffs	_((numdata *npt1, numdata *npt2));
 PUBLIC int normfloat	_((int prec, numdata *nptr));
 PUBLIC int multtrunc	_((int prec, numdata *npt1, numdata *npt2,
@@ -65,6 +67,7 @@ PUBLIC void int2numdat	_((int x, numdata *nptr));
 PUBLIC void cpynumdat	_((numdata *npt1, numdata *npt2));
 PUBLIC int numposneg	_((truc *ptr));
 PUBLIC truc wipesign	_((truc *ptr));
+PUBLIC truc changesign	_((truc *ptr));
 PUBLIC long intretr	_((truc *ptr));
 PUBLIC int bigref	_((truc *ptr, word2 **xp, int *sp));
 PUBLIC int bigretr	_((truc *ptr, word2 *x, int *sp));
@@ -72,11 +75,16 @@ PUBLIC int twocretr	_((truc *ptr, word2 *x));
 PUBLIC int and2arr	_((word2 *x, int n, word2 *y, int m));
 PUBLIC int or2arr	_((word2 *x, int n, word2 *y, int m));
 PUBLIC int xor2arr	_((word2 *x, int n, word2 *y, int m));
+PUBLIC int xorbitvec	_((word2 *x, int n, word2 *y, int m));
 PUBLIC long bit_length	_((word2 *x, int n));
+PUBLIC int chkintnz _((truc sym, truc *ptr));
 PUBLIC int chkints	_((truc sym, truc *argptr, int n));
+PUBLIC int chkint   _((truc sym, truc *ptr));
+PUBLIC int chkintt  _((truc sym, truc *ptr));
 PUBLIC int chknums	_((truc sym, truc *argptr, int n));
 PUBLIC int chknum	_((truc sym, truc *ptr));
-
+PUBLIC int chkintvec	_((truc sym, truc *vptr));
+PUBLIC int chknumvec	_((truc sym, truc *vptr));
 
 PRIVATE long decdigs	_((numdata *nptr));
 PRIVATE int malzehnhoch _((int prec, numdata *nptr, long d, word2 *hilf));
@@ -85,7 +93,12 @@ PRIVATE int twocadjust	_((word2 *x, int n));
 PUBLIC int FltPrec[20] = 
 	{2, 4, 8, 12, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 
 	160, 192, 224, 256, 320};
+
+#ifdef FPREC_HIGH
 #define MAXFLTLEVEL 18
+#else
+#define MAXFLTLEVEL 10
+#endif
 
 PUBLIC int MaxFltLevel = MAXFLTLEVEL;
 
@@ -727,6 +740,32 @@ truc *ptr;
 }
 /*------------------------------------------------------------------*/
 /*
+** Aendert destruktiv das Vorzeichen der in *ptr gegebenen Zahl.
+*/
+PUBLIC truc changesign(ptr)
+truc *ptr;
+{
+    int sign, flg;
+
+    sign = numposneg(ptr);
+    if(sign == 0)
+        return *ptr;
+    else if(sign < 0)
+        return wipesign(ptr);
+
+    /* now *ptr is positive */
+	flg = *FLAGPTR(ptr);
+	if(flg == fFIXNUM)
+		*SIGNPTR(ptr) = MINUSBYTE;
+	else if(flg >= fFLTOBJ) {
+			*SIGNUMPTR(ptr) |= FSIGNBIT;
+	}
+	else		/* bignums */
+		*SIGNUMPTR(ptr) = MINUSBYTE;
+	return(*ptr);
+}
+/*------------------------------------------------------------------*/
+/*
 ** holt long aus *ptr;
 ** falls abs(Zahl) >= 2 ** 31, Returnwert: LONGERROR
 */
@@ -746,7 +785,7 @@ truc *ptr;
 }
 /*------------------------------------------------------------------*/
 /*
-** Erzeugt Referenz auf Integer (bignum oder fixnum) ohne Kopie
+** Erzeugt Referenz auf Integer (bignum, fixnum oder gf2n_int) ohne Kopie
 ** Rueckgabe aERROR, falls *ptr kein fixnum oder bignum
 */
 PUBLIC int bigref(ptr,xp,sp)
@@ -763,13 +802,13 @@ int *sp;
 		*sp = *SIGNPTR(ptr);
 		return(*x ? 1 : 0);
 	}
-	else if(flg == fBIGNUM) {
+	else if(flg == fBIGNUM || flg == fGF2NINT) {
 		big = (struct bigcell *)TAddress(ptr);
 		*sp = big->signum;
 		*xp = &(big->digi0);
 		return(big->len);
 	}
-	else 
+	else
 		return(aERROR);
 }
 /*------------------------------------------------------------------*/
@@ -892,6 +931,24 @@ int n,m;
 	return(n);
 }
 /*-------------------------------------------------------------*/
+/*
+** Bitwise xor of two bitvectors (x,n), (y,m) 
+** Destructively replaces (x,n) by result
+*/
+PUBLIC int xorbitvec(x,n,y,m)
+word2 *x,*y;
+int n,m;
+{
+	if(n < m) {
+		setarr(x+n,m-n,0);
+		n = m;
+	}
+	xorarr(x,m,y);
+	while(n > 0 && x[n-1] == 0)
+		n--;
+	return(n);
+}
+/*-------------------------------------------------------------*/
 PUBLIC long bit_length(x,n)
 word2 *x;
 int n;
@@ -929,6 +986,89 @@ int n;
 	}
 	return(flg0);
 }
+/*------------------------------------------------------------------*/
+/*
+** checks whether element *ptr is an integer
+** Return value: fFIXNUM, fBIGNUM or aERROR
+*/
+PUBLIC int chkint(sym,ptr)
+truc sym;
+truc *ptr;
+{
+    int flg;
+
+    flg = *FLAGPTR(ptr);
+	if(flg < fFIXNUM || flg > fBIGNUM)
+		return(error(sym,err_int,*ptr));
+    else
+        return flg;
+}
+/*------------------------------------------------------------------*/
+/*
+** checks whether element *ptr is an integer or gf2nint
+** Return value: fFIXNUM, fBIGNUM, fGF2NINT or aERROR
+*/
+PUBLIC int chkintt(sym,ptr)
+truc sym;
+truc *ptr;
+{
+    int flg;
+
+    flg = *FLAGPTR(ptr);
+	if(flg < fINTTYPE0 || flg > fINTTYPE1)
+		return(error(sym,err_intt,*ptr));
+    else
+        return flg;
+}
+/*------------------------------------------------------------------*/
+/*
+** checks whether element *ptr is a nonzero integer
+** Return value: fFIXNUM, fBIGNUM or aERROR
+*/
+PUBLIC int chkintnz(sym,ptr)
+truc sym;
+truc *ptr;
+{
+    int flg;
+
+    flg = *FLAGPTR(ptr);
+	if(flg < fFIXNUM || flg > fBIGNUM)
+		return(error(sym,err_int,*ptr));
+    else if(*ptr == zero)
+        return(error(sym,err_div,voidsym));
+    else
+        return flg;
+}
+/*-------------------------------------------------------------*/
+/*
+** Argument *vptr must be a vector.
+** The function checks whether all components of this
+** vector are integers.
+** Return value: fFIXNUM, fBIGNUM or aERROR
+*/
+PUBLIC int chkintvec(sym,vptr)
+truc sym;
+truc *vptr;
+{
+	struct vector *vec;
+	truc *ptr;
+	int len;
+	int flg, flg0 = fFIXNUM;
+
+	vec = (struct vector *)TAddress(vptr);
+	len = vec->len;
+	ptr = &(vec->ele0);
+
+	while(--len >= 0) {
+		flg = *FLAGPTR(ptr);
+		if(flg < fFIXNUM || flg > fBIGNUM)
+			return(error(sym,err_int,*ptr));
+		else if(flg > flg0)
+			flg0 = flg;
+		ptr++;
+	}
+	return(flg0);
+}
 /*-------------------------------------------------------------*/
 /*
 ** Prueft, ob die Elemente argptr[i], 0 <= i < n, Integers oder
@@ -963,4 +1103,34 @@ truc *ptr;
 		return(error(sym,err_num,*ptr));
 	return(flg);
 }
+/*-------------------------------------------------------------*/
+/*
+** Argument *vptr must be a vector.
+** The function checks whether all components of this
+** vector are integers.
+** Return value: fFIXNUM, fBIGNUM, Float-flag or aERROR
+*/
+PUBLIC int chknumvec(sym,vptr)
+truc sym;
+truc *vptr;
+{
+	struct vector *vec;
+	truc *ptr;
+	int len;
+	int flg, flg0 = fFIXNUM;
+
+	vec = (struct vector *)TAddress(vptr);
+	len = vec->len;
+	ptr = &(vec->ele0);
+
+	while(--len >= 0) {
+		flg = *FLAGPTR(ptr);
+		if(flg < fFIXNUM)
+			return(error(sym,err_num,*ptr));
+		else if(flg > flg0)
+			flg0 = flg;
+		ptr++;
+	}
+	return(flg0);
+}
 /***************************************************************/
