@@ -43,12 +43,11 @@ Email	forster@mathematik.uni-muenchen.de
 ** 2002-04-19:  divide(x,y)
 ** 2003-02-11:  addition of gf2n_int's
 ** 2003-02-15:  Fdecode made safe wrt garbage collection
+** 2003-11-21:  removed bug in vecdiv
 */
 
 #include "common.h"
 
-#define ARITZ
-
 PUBLIC void iniarith	_((void));
 PUBLIC truc addints		_((truc *ptr, int minflg));
 PUBLIC truc scalintvec 	_((truc *ptr1, truc *ptr2));
@@ -278,9 +277,7 @@ PUBLIC void iniarith()
 	inirandstate(RandSeed);
 	iniaritx();
 	iniarity();
-#ifdef ARITZ
 	iniaritz();
-#endif
 }
 /*--------------------------------------------------------*/
 #define DIVFLAG		1
@@ -535,6 +532,9 @@ PRIVATE truc Ftimes()
 	return(res);
 }
 /*----------------------------------------------------------*/
+/*
+** multiply integers in ptr[0] and ptr[1]
+*/
 PRIVATE truc multints(ptr)
 truc *ptr;
 {
@@ -1137,7 +1137,7 @@ truc *zz;
 		else {
 			sign1 = MINUSBYTE;
 			if(rlen)
-				len = incarr(AriBuf,len,1);
+				len1 = incarr(AriBuf,len1,1);
 		}
 		obj = mkint(sign1,AriBuf,len1);
 		*(VECTORPTR(workStkPtr)+k) = obj;
