@@ -49,7 +49,7 @@ Email	forster@mathematik.uni-muenchen.de
 ** 1999-06-03:  power for exponents a >= 2**16; return to old version of modbig
 ** 2002-04-20:  modmultbig, addsarr
 ** 2003-06-16:  modnegbig
-** 2003-11-07:  returning to old version of divbig (new version was buggy)
+** 2003-11-09:  bugfix in function divbig
 */
 
 #include "common.h"
@@ -414,7 +414,7 @@ PUBLIC int modnegbig(x,n,y,m,hilf)
 word2 *x, *y, *hilf;
 int n, m;
 {
-    int len, cmp;
+    int cmp;
 
     cmp = cmparr(y,m,x,n);
     if(cmp == 0)
@@ -442,8 +442,7 @@ int xlen, ylen, mlen;
     return len;
 }
 /*-------------------------------------------------------------------*/
-/* reinstalled old version 2003-11-08 */
-#if 1
+#ifdef OLDVERSION
 /*
 ** quot = (x,n) / (y,m); Voraussetzung (y,m) != null
 ** Arbeitet destruktiv auf x; x wird rest, seine Laenge in *rlenptr
@@ -525,9 +524,8 @@ int n, m, *rlenptr;
 	*rlenptr = n;
 	return(quotlen);
 }
-#endif
 /*-------------------------------------------------------------------*/
-#ifdef BUGGYVERSION
+#else /* NEWVERSION */
 /*
 ** quot = (x,n) / (y,m); Voraussetzung (y,m) != null
 ** Arbeitet destruktiv auf x; x wird rest, seine Laenge in *rlenptr
@@ -612,11 +610,11 @@ int n, m, *rlenptr;
 		    u++;
 		    n = subarr(x+kappa,n,y,m);
 		}
-		quot[kappa+1] = (u >> 16);
-		quot[kappa] = u;
+		quot[kappa] = (word2)u;
+		quot[kappa+1] = (word2)(u >> 16);
 		if(n == 0) {
 		    n = kappa;
-		    while(n > 0 && x[n-1])
+		    while(n > 0 && !x[n-1])
                 n--;
 		}
 		else
@@ -677,7 +675,7 @@ int n, m, *rlenptr;
 	return(quotlen);
 #endif
 }
-#endif BUGGYVERSION
+#endif /* ?OLDVERSION */
 /*-------------------------------------------------------------------*/
 /*
 ** Berechnet Produkt von (x,n)*(2^16)^-prec mit (y,m)*(2^16)^-prec
