@@ -2,7 +2,7 @@
 /* file aritx.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996/2003 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 /*
@@ -33,18 +33,20 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** extended arithmetic functions
 **
 ** date of last change
-** 95-01-07:	brill1
-** 95-03-05:	ithbit
-** 95-03-11:	mod_pemult, mod_coshmult
-** 95-04-09:	rab_primetest
-** 97-02-24:	rab_primetest, overflow handling
-** 97-03-02:	cf_factorize: small speedup (pivots)
-** 97-04-13:	reorg (newintsym)
-** 97-11-26:	fixed bug in Ffactorial
-** 97-12-09:	factorization routines moved to arity.c
-** 97-12-14:	changed function prime32
-** 98-10-17:    small change in gcdcx
-** 98-10-25:    allow interrupt in modpower
+** 1995-01-07:	brill1
+** 1995-03-05:	ithbit
+** 1995-03-11:	mod_pemult, mod_coshmult
+** 1995-04-09:	rab_primetest
+** 1997-02-24:	rab_primetest, overflow handling
+** 1997-03-02:	cf_factorize: small speedup (pivots)
+** 1997-04-13:	reorg (newintsym)
+** 1997-11-26:	fixed bug in Ffactorial
+** 1997-12-09:	factorization routines moved to arity.c
+** 1997-12-14:	changed function prime32
+** 1998-10-17:  small change in gcdcx
+** 1998-10-25:  allow interrupt in modpower
+** 2002-02-24:  corrected sign in modpower when modulus negative
+** 2003-03-01:  allow negative exponents in modpower
 */
 
 #include "common.h"
@@ -59,16 +61,22 @@ PUBLIC truc modpowsym;
 /*-------------------------------------------------------------------*/
 PUBLIC int prime16	_((unsigned u));
 PUBLIC int prime32	_((word4 u));
+PUBLIC unsigned fact16    _((word4 u));
 PUBLIC unsigned trialdiv  _((word2 *x, int n, unsigned u0, unsigned u1));
 PUBLIC int jacobi	_((int sign, word2 *x, int n, word2 *y, int m,
 			   word2 *hilf));
 PUBLIC int jac		_((unsigned x, unsigned y));
 PUBLIC int rabtest	_((word2 *x, int n, word2 *aux));
+PUBLIC int nextprime32 _((word4 u, word2 *x));
+PUBLIC int pemult	_((word2 *x, int n, word2 *ex, int exlen,
+			   word2 *aa, int alen,
+			   word2 *mm, int modlen, word2 *z, word2 *hilf));
 PUBLIC int modinverse	_((word2 *x, int n, word2 *y, int m, word2 *zz,
 			   word2 *hilf));
+PUBLIC int modinv   _((int x, int mm));
 PUBLIC int modpower	_((word2 *x, int n, word2 *ex, int exlen,
 			   word2 *mm, int modlen, word2 *p, word2 *hilf));
-
+PUBLIC unsigned modpow    _((unsigned x, unsigned n, unsigned mm));
 
 PRIVATE void primsiev	_((word2 *vect, int n));
 PRIVATE truc Fjacobi	_((void));
@@ -76,7 +84,9 @@ PRIVATE truc Fjacobi	_((void));
 PRIVATE truc Ffact16	_((int argn));
 PRIVATE truc Fprime32	_((void));
 PRIVATE truc Frabtest	_((void));
-PRIVATE int rabtest0	_((word2 *x, int n, unsigned u, word2 *aux));
+PRIVATE int rabtest0 	_((word2 *x, int n, unsigned u, word2 *aux));
+
+PRIVATE truc Fnextprime _((int argn));
 
 PRIVATE truc Fgcd	_((int argn));
 PRIVATE unsigned gcdfixnums  _((truc *argptr, int argn));
@@ -93,36 +103,33 @@ PRIVATE truc Fcoshmult	_((int argn));
 PRIVATE int coshmult	_((word2 *x, int n, word2 *ex, int exlen,
 			   word2 *mm, int modlen, word2 *z, word2 *hilf));
 PRIVATE truc Fpemult	_((int argn));
-PRIVATE int pemult	_((word2 *x, int n, word2 *ex, int exlen,
-			   word2 *aa, int alen,
-			   word2 *mm, int modlen, word2 *z, word2 *hilf));
-
 PRIVATE truc Ffactorial	 _((void));
 PRIVATE int factorial	_((unsigned a, word2 *x));
 
 PRIVATE truc jacobsym, gcdsym, gcdxsym, modinvsym;
 PRIVATE truc factorsym, primsym, rabtestsym;
+PRIVATE truc nxtprimsym;
 PRIVATE truc pemultsym, cshmultsym;
 PRIVATE truc factsym;
 
 /*------------------------------------------------------------------*/
 PUBLIC void iniaritx()
 {
-	modpowsym = newintsym("** mod", sFBINARY,(truc)Fmodpower);
+	modpowsym = newintsym("** mod", sFBINARY,(wtruc)Fmodpower);
 
-	gcdsym	  = newsymsig("gcd",sFBINARY,(truc)Fgcd, s_0uii);
-	gcdxsym	  = newsymsig("gcdx",sSBINARY,(truc)Sgcdx, s_iiiII);
-	modinvsym = newsymsig("mod_inverse",sFBINARY,(truc)Fmodinv, s_iii);
+	gcdsym	  = newsymsig("gcd",sFBINARY,(wtruc)Fgcd, s_0uii);
+	gcdxsym	  = newsymsig("gcdx",sSBINARY,(wtruc)Sgcdx, s_iiiII);
+	modinvsym = newsymsig("mod_inverse",sFBINARY,(wtruc)Fmodinv, s_iii);
 
-	jacobsym  = newsymsig("jacobi",	 sFBINARY,   (truc)Fjacobi, s_iii);
-	primsym	  = newsymsig("prime32test",sFBINARY,(truc)Fprime32,s_ii);
-	rabtestsym= newsymsig("rab_primetest",sFBINARY,(truc)Frabtest,s_1);
-	factorsym = newsymsig("factor16",sFBINARY,(truc)Ffact16, s_13);
+	jacobsym  = newsymsig("jacobi",	 sFBINARY,   (wtruc)Fjacobi, s_iii);
+	primsym	  = newsymsig("prime32test",sFBINARY,(wtruc)Fprime32,s_ii);
+	rabtestsym= newsymsig("rab_primetest",sFBINARY,(wtruc)Frabtest,s_1);
+    nxtprimsym= newsymsig("next_prime",sFBINARY,(wtruc)Fnextprime, s_12);
+	factorsym = newsymsig("factor16",sFBINARY,(wtruc)Ffact16, s_13);
 
-	pemultsym = newsymsig("mod_pemult",sFBINARY,(truc)Fpemult, s_Viiii);
-	cshmultsym= newsymsig("mod_coshmult",sFBINARY,(truc)Fcoshmult,
-						s_iiii);
-	factsym	  = newsymsig("factorial", sFBINARY, (truc)Ffactorial,s_ii);
+	pemultsym = newsymsig("mod_pemult",sFBINARY,(wtruc)Fpemult, s_Viiii);
+	cshmultsym= newsymsig("mod_coshmult",sFBINARY,(wtruc)Fcoshmult, s_iiii);
+	factsym	  = newsymsig("factorial", sFBINARY, (wtruc)Ffactorial,s_ii);
 
 	primsiev(PrimTab,PRIMTABSIZE);
 }
@@ -196,6 +203,122 @@ word4 u;
 	}
 	return(1);
 }
+/*-------------------------------------------------------------------*/
+/*
+** Rueckgabewert 0, falls u Primzahl ist,
+** sonst kleinster Primteiler
+*/
+PUBLIC unsigned fact16(u)
+word4 u;
+{
+	unsigned mask, v, p;
+	word2 *pbits;
+
+
+	if(!(u & 1))
+		return(2);
+
+	v = intsqrt(u);
+	pbits = PrimTab;
+	mask = 2;
+	for(p=3; p<=v; p += 2) {
+		if((*pbits & mask) && (u % p == 0))
+			return(p);
+		mask <<= 1;
+		if((mask & 0xFFFF) == 0) {
+			pbits++;
+			mask = 1;
+		}
+	}
+	return(0);
+}
+/*------------------------------------------------------------------*/
+PUBLIC int nextprime32(u,x)
+word4 u;
+word2 *x;
+{
+    int n;
+
+    if(u <= 2) {
+        *x = 2;
+        return(1);
+    }
+    else if(u <= 0xFFFFFFFB) {
+        if((u & 1) == 0)
+            u++;
+        while(prime32(u) == 0)
+            u += 2;
+        n = long2big(u,x);
+        return(n);
+    }
+    else {
+        x[0] = 0xF;
+        x[1] = 0;
+        x[2] = 1;
+        return(3);
+    }
+}
+/*------------------------------------------------------------------*/
+PRIVATE truc Fnextprime(argn)
+int argn;
+{
+#define ANZRAB  10
+    truc *argptr;
+    word2 *x;
+    word4 u;
+    int compos;
+    int i, n, sign;
+    int doreport;
+
+    argptr = argStkPtr-argn+1;
+    if(argn >= 2 && *argStkPtr == zero) {
+        doreport = 0;
+        argn--;
+    }
+    else {
+        doreport = 1;
+    }
+    n = bigref(argptr,&x,&sign);
+    if(n == aERROR) {
+        error(nxtprimsym,err_int,*argptr);
+        return(brkerr());
+    }
+    if(n >= aribufSize/9) {
+        error(nxtprimsym,err_int,*argStkPtr);
+        return(brkerr());
+    }
+    if(n <= 2) {
+        u = big2long(x,n);
+        n = nextprime32(u,AriBuf);
+    }
+    else {
+        cpyarr(x,n,AriBuf);
+        x = AriBuf;
+        if((x[0] & 1) == 0)
+            x[0]++;
+        compos = 1;
+        if(doreport)
+            workmess();
+        while(compos) {
+            while(trialdiv(x,n,3,0xFFFB))
+                n = incarr(x,n,2);
+            for(compos=0,i=0; i<ANZRAB; i++) {
+                if(rabtest(x,n,AriScratch) == 0) {
+                    compos = 1;
+                    if(doreport)
+                        tick('.');
+                    n = incarr(x,n,2);
+                    break;
+                }
+                if(doreport)
+                    tick(',');
+            }
+        }
+        if(doreport)
+            fprintstr(tstdout," probable prime:");
+    }
+    return(mkint(0,AriBuf,n));
+}
 /*------------------------------------------------------------------*/
 PRIVATE truc Fjacobi()
 {
@@ -458,7 +581,7 @@ PRIVATE truc Frabtest()
 */
 	word2 *x;
 	int sign, n;
-        unsigned u;
+	unsigned u;
 
 	n = bigref(argStkPtr,&x,&sign);
 	if(n == aERROR) {
@@ -473,9 +596,9 @@ PRIVATE truc Frabtest()
 	else if((x[0] & 1) == 0)
 		return(false);
 	else if(n < scrbufSize/9 && n < aribufSize/2) {
-                u = 2 + random2(64000);
+		u = 2 + random2(64000);
 		return(rabtest0(x,n,u,AriScratch) ? true : false);
-        }
+	}
 	else {
 		error(rabtestsym,err_ovfl,*argStkPtr);
 		return(brkerr());
@@ -484,6 +607,7 @@ PRIVATE truc Frabtest()
 /*------------------------------------------------------------------*/
 /*
 ** Puffer aux muss mindestens 8*n + 2 lang sein.
+** u is a positive integer < 2**16
 */
 PRIVATE int rabtest0(x,n,u,aux)
 word2 *x, *aux;
@@ -529,10 +653,10 @@ PUBLIC int rabtest(x,n,aux)
 word2 *x, *aux;
 int n;
 {
-        unsigned u;
+	unsigned u;
 
-        u = 2 + random2(64000);
-        return(rabtest0(x,n,u,aux));
+	u = 2 + random2(64000);
+	return(rabtest0(x,n,u,aux));
 }
 /*------------------------------------------------------------------*/
 PRIVATE truc Fgcd(argn)
@@ -542,19 +666,28 @@ int argn;		/* 0 <= argn < infinity */
 	unsigned u;
 	int n, flg;
 
-	if(argn == 0)
+	if(argn == 1 && *FLAGPTR(argStkPtr) == fVECTOR) {
+		flg = chkintvec(gcdsym,argStkPtr);
+		if(flg >= fFIXNUM) {
+			argn = *VECLENPTR(argStkPtr);
+			argptr = VECTORPTR(argStkPtr);
+		}
+    }
+	else if(argn > 0) {
+		argptr = argStkPtr - argn + 1;
+		flg = chkints(gcdsym,argptr,argn);
+    }
+	else if(argn == 0)
 		return(zero);
-	argptr = argStkPtr - argn + 1;
-	flg = chkints(gcdsym,argptr,argn);
+
 	if(flg == aERROR)
 		return(brkerr());
-	if(argn == 1)
-		return(*argStkPtr);
+
 	if(flg == fFIXNUM) {
 		u = gcdfixnums(argptr,argn);
 		return(mkfixnum(u));
 	}
-	else {
+	else {	/* flg == fBIGNUM */
 		n = gcdbignums(AriBuf,AriScratch,argptr,argn);
 		return(mkint(0,AriBuf,n));
 	}
@@ -709,17 +842,17 @@ int *cxlptr;
 	q = hilf + N;
 	alfa = hilf + 2*N;
 	prod = hilf + 3*N;
-        hilf += 4*N;
+    hilf += 4*N;
 
 	if(m == 0) {
 		return(0);
 	}
 	n2 = modbig(x,n,y,m,hilf);
-        if(n2 == 0) {
-                cpyarr(y,m,x);
-                *cxlptr = 0;
-                return(m);
-        }
+    if(n2 == 0) {
+        cpyarr(y,m,x);
+        *cxlptr = 0;
+        return(m);
+    }
 	x2 = x;
 	cpyarr(y,m,x1);
 	n1 = m;
@@ -731,9 +864,9 @@ int *cxlptr;
 ** (x1,n1) = -+(alfa,alen)*(x,n) mod (y,m)
 ** (x2,n2) = +-(beta,blen)*(x,n) mod (y,m)
 */
-        count = 0;
+    count = 0;
 	while(qlen = divbig(x1,n1,x2,n2,q,&rlen,hilf), rlen) {
-                count++;
+        count++;
 		/* x1neu = x2alt, x2neu = rest */
 		temp = x1;
 		x1 = x2;
@@ -762,6 +895,7 @@ int *cxlptr;
 /*
 ** Berechnet das Inverse von (x,n) modulo (y,m)
 ** Resultat wird in zz abgelegt, seine Laenge ist Rueckgabewert.
+** x und y bleiben erhalten
 ** Falls x und y nicht teilerfremd, wird 0 zurueckgegeben.
 */
 PUBLIC int modinverse(x,n,y,m,zz,hilf)
@@ -781,6 +915,33 @@ int n, m;
 	else
 		return(len);
 }
+/*---------------------------------------------------------------*/
+/*
+** Calculates inverse of x mod mm
+** If x is not invertible mod mm, then 0 is returned
+*/
+PUBLIC int modinv(x,mm)
+int x,mm;
+{
+    int y, yold, q, q1, q2, q2old;
+
+   	q1 = 1; q2 = 0;
+	y = mm;
+    while(y) {
+		yold = y;
+		q = x / y;
+		y = x % y;
+		x = yold;
+		q2old = q2;
+		q2 = q1 - q*q2;
+		q1 = q2old;
+    }
+    if(x == 1) {
+        return (q1 >= 0 ? q1 : mm+q1);
+    }
+    else
+        return 0;
+}
 /*------------------------------------------------------------------*/
 /*
 ** Berechnet den Koeffizienten cy fuer die Darstellung
@@ -800,14 +961,14 @@ int n1, n, m, cxlen;
 	temp = hilf;
 	hilf += n + cxlen + 2;
 
-        if(cxlen) {
-	        tlen = multbig(x,n,cx,cxlen,temp,hilf);
-	        tlen = subarr(temp,tlen,x1,n1);
-        }
-        else {
-                cpyarr(x1,n1,temp);
-                tlen = n1;
-        }
+    if(cxlen) {
+        tlen = multbig(x,n,cx,cxlen,temp,hilf);
+        tlen = subarr(temp,tlen,x1,n1);
+    }
+    else {
+        cpyarr(x1,n1,temp);
+        tlen = n1;
+    }
 	len = divbig(temp,tlen,y,m,cy,&rlen,hilf);
 	return(len);
 }
@@ -818,7 +979,7 @@ int n1, n, m, cxlen;
 PRIVATE truc Fmodpower(argn)
 int argn;
 {
-	word2 *x, *y, *z, *p, *hilf;
+	word2 *x, *y, *z, *hilf;
 	int n, n1, n2, n3;
 	int sign1, sign2, sign3;
 
@@ -826,30 +987,49 @@ int argn;
 		return(brkerr());
 
 	x = AriScratch;
-	p = AriBuf;
 	hilf = AriScratch + aribufSize;
 
-	n2 = bigref(argStkPtr-1,&y,&sign2);
-	if(sign2) {
-		error(modpowsym,err_pnum,*(argStkPtr-1));
-		return(brkerr());
-	}
 	n1 = bigretr(argStkPtr-2,x,&sign1);
+	n2 = bigref(argStkPtr-1,&y,&sign2);
 	n3 = bigref(argStkPtr,&z,&sign3);
 	if(!n3) {
 		error(modpowsym,err_div,voidsym);
 		return(brkerr());
 	}
-        /* overflow? */
-        n = (n3 > n1 ? n3 : n1) + 3;
-        if (n >= aribufSize/2 || n + aribufSize/3 >= scrbufSize/3) {
-                error(modpowsym,err_ovfl,voidsym);
+	/* overflow? */
+	n = (n3 > n1 ? n3 : n1) + 3;
+	if (n >= aribufSize/2 || n + aribufSize/3 >= scrbufSize/3) {
+		error(modpowsym,err_ovfl,voidsym);
 		return(brkerr());
+	}
+	if(sign2) {
+        if(n1)
+            n1 = modinverse(x,n1,z,n3,AriBuf,hilf);
+        if(!n1) {
+		    error(modpowsym,err_div,voidsym);
+		    return(brkerr());
         }
-	n = modpower(x,n1,y,n2,z,n3,p,hilf);
-	if(sign1 && n && n2 && (*y & 1))
-		n = sub1arr(p,n,z,n3);
-	return(mkint(0,p,n));
+        if(n2 == 1 && y[0] == 1) {
+            n = n1;
+            goto testsigns;
+        }
+        else {
+            cpyarr(AriBuf,n1,x);
+        }
+	}
+	n = modpower(x,n1,y,n2,z,n3,AriBuf,hilf);
+	if(n == 0)
+		return zero;
+	/* else */
+	if(n2 == 0)
+		sign1 = 0;
+	else if((*y & 1) == 0)
+		sign1 = 0;
+  testsigns:
+	if(sign1 != sign3)
+		n = sub1arr(AriBuf,n,z,n3);
+
+	return(mkint(sign3,AriBuf,n));
 }
 /*-------------------------------------------------------------------*/
 /*
@@ -865,7 +1045,7 @@ int n, exlen, modlen;
 {
 	word2 *temp;
 	int k, plen;
-        int allowintr;
+    int allowintr;
 
 	temp = hilf + 2*modlen + 2;
 
@@ -881,7 +1061,7 @@ int n, exlen, modlen;
 	plen = n;		/* plen <= modlen */
 
 	k = ((exlen-1) << 4) + bitlen(ex[exlen-1]) - 1;
-        allowintr = (modlen >= 16 && (k/16 + modlen >= 256) ? 1 : 0);
+    allowintr = (modlen >= 16 && (k/16 + modlen >= 256) ? 1 : 0);
 	while(--k >= 0) {
 		plen = multbig(p,plen,p,plen,temp,hilf);
 		cpyarr(temp,plen,p);
@@ -891,13 +1071,34 @@ int n, exlen, modlen;
 			cpyarr(temp,plen,p);
 			plen = modbig(p,plen,mm,modlen,hilf);
 		}
-                if(allowintr && INTERRUPT) {
-                        setinterrupt(0);
-                        reset(err_intr);
-                }
+        if(allowintr && INTERRUPT) {
+            setinterrupt(0);
+            reset(err_intr);
+        }
 	}
 	return(plen);
 }
+/*---------------------------------------------------------------*/
+/*
+** Calculates x**n mod mm
+** mm must be < 2**16
+*/
+PUBLIC unsigned modpow(x,n,mm)
+unsigned x,n,mm;
+{
+    word4 u,z;
+
+    if(n == 0)
+        return(1);
+    z = 1; u = x % mm;
+    while(n > 1) {
+        if(n & 1)
+            z = (z*u) % mm;
+        u = (u*u) % mm;
+        n >>= 1;
+    }
+    return((z*u) % mm);
+}
 /*------------------------------------------------------------------*/
 /*
 ** mod_coshmult(Base,Exponent,Module)
@@ -1044,7 +1245,7 @@ int argn;	/* argn = 4 */
 ** mm auftaucht, wird dieser Teiler in z abgelegt;
 ** der Rueckgabewert ist dann -(lenz + 1).
 */
-PRIVATE int pemult(x,n,ex,exlen,aa,alen,mm,modlen,z,hilf)
+PUBLIC int pemult(x,n,ex,exlen,aa,alen,mm,modlen,z,hilf)
 word2 *x, *ex, *aa, *mm, *z, *hilf;
 int n, exlen, alen, modlen;
 {
