@@ -40,6 +40,7 @@ Email   forster@mathematik.uni-muenchen.de
 ** 1998-10-11:	function next_prime
 ** 2000-04-26:	fixed bug in ATARI version due to ptr diffs > 32K
 ** 2003-05-30:  fixed bug in qs_factorize
+** 2004-08-20:  Removed #define ETEST
 */
 
 #include "common.h"
@@ -203,7 +204,6 @@ typedef struct {
     int ylen;
 } EPOINT;
 /*-----------------------------------------------------------------*/
-#define ETEST
 
 #ifdef QTEST    /* only for testing */
 #define DBGFILE
@@ -331,6 +331,7 @@ PUBLIC void iniarity()
 #endif
 }
 /*-------------------------------------------------------------------*/
+#ifdef ETEST
 PRIVATE truc Feee()
 {
     word2 *N, *aa, *xx;
@@ -407,7 +408,7 @@ PRIVATE truc Fee2()
     ECN ecN;
     EPOINT PP;
 
-#ifdef ETEST
+#ifdef ETEST1
 dbgf = fopen("etest.log","w");
 #endif
 	if(chkints(eeesym,argStkPtr-3,4) == aERROR)
@@ -443,7 +444,7 @@ dbgf = fopen("etest.log","w");
 
     len = ECNmult(&ecN,&PP,ex,exlen,hilf);
 
-#ifdef ETEST
+#ifdef ETEST1
 fclose(dbgf);
 #endif
 
@@ -452,6 +453,7 @@ fclose(dbgf);
     else
         return mkinum(len);
 }
+#endif /* ETEST */
 /*-------------------------------------------------------------------*/
 /*
 ** Messages for factorization algorithms
