@@ -2,7 +2,7 @@
 /* file aritz.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996-2002 O.Forster
+Copyright (C) 1996-2004 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -148,6 +148,8 @@ PRIVATE truc Fggg	_((void));
 PRIVATE truc Fgg1	_((void));
 PRIVATE truc Fgg2	_((void));
 #endif
+
+#define GF2NINTEGER
 /*------------------------------------------------------------------*/
 PUBLIC void iniaritz()
 {
@@ -165,18 +167,19 @@ PUBLIC void iniaritz()
     gf2ndegsym = newsymsig("gf2n_degree", sFBINARY, (wtruc)Fgf2ndegree, s_0);
     gf2npolsym = newsymsig("gf2n_fieldpol",sFBINARY,(wtruc)Fgf2nfieldpol,s_0);
     maxgf2nsym = newsymsig("max_gf2nsize",sFBINARY, (wtruc)Fmaxgf2n, s_0);
-#endif /* GF2NINTEGER */
+#endif GF2NINTEGER
 
 #ifdef POLYARITH
     polmultsym = newsymsig("pol_mult",  sFBINARY, (wtruc)Fpolmult, s_2);
-   	polNmultsym= newintsym("pol_mult() mod", sFBINARY,(wtruc)FpolNmult);
+    polNmultsym= newintsym("pol_mult() mod", sFBINARY,(wtruc)FpolNmult);
     polmodsym  = newsymsig("pol_mod", sFBINARY, (wtruc)Fpolmod, s_2);
-   	polNmodsym = newintsym("pol_mod() mod", sFBINARY,(wtruc)FpolNmod);
+    polNmodsym = newintsym("pol_mod() mod", sFBINARY,(wtruc)FpolNmod);
     poldivsym  = newsymsig("pol_div", sFBINARY, (wtruc)Fpoldiv, s_2);
-   	polNdivsym = newintsym("pol_div() mod", sFBINARY,(wtruc)FpolNdiv);
+    polNdivsym = newintsym("pol_div() mod", sFBINARY,(wtruc)FpolNdiv);
 #endif /* POLYARITH */
 
     fpsqrtsym = newsymsig("gfp_sqrt", sFBINARY, (wtruc)Ffpsqrt, s_2);
+
 #ifdef NAUSKOMM
 	gggsym = newsymsig("ggg", sFBINARY, (wtruc)Fggg, s_2);
 	gg1sym = newsymsig("gg1", sFBINARY, (wtruc)Fgg1, s_2);
@@ -1352,6 +1355,9 @@ truc *ptr;
     return mkgf2n(z,N);
 }
 /*-------------------------------------------------------------------*/
+/*
+** Transforms object in *argStkPtr to data type gf2nint
+*/
 PRIVATE truc Fgf2nint()
 {
 	word2 *x;
@@ -1702,13 +1708,17 @@ int n;
 
     if(n == 0)
         return 0;
-    N = 2*n - 1;
+    N = 2*n - 2;
     bptr = (byte *)x;
     for(k=0; k<N; k++) {
         u = *bptr++;
         z[k] = spreadbyte[u];
     }
-    if(u = *bptr) {
+    u = x[n-1];
+    z[N] = spreadbyte[u & 0x00FF];
+    N++;
+    u >>= 8;
+    if(u) {
         z[N] = spreadbyte[u];
         N++;
     }
@@ -1864,7 +1874,7 @@ int *zlenptr;
 /*-----------------------------------------------------------------*/
 /*
 ** Tests whether the gf2 polynomial given by (x,n) is irreducible.
-** yy and zz are auxiliary array needed for intermediate calculations
+** yy and zz are auxiliary arrays needed for intermediate calculations
 */
 PRIVATE int gf2polirred(x,n,yy,zz)
 word2 *x, *yy, *zz;
