@@ -34,22 +34,31 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** array, string, byte_string, record and stack procedures
 **
 ** date of last change
-** 95-03-05:	Findex changed
-** 95-03-20:	ARGV sSYSTEMVAR
-** 95-04-03:	records, pointer
-** 95-04-07:	alloc
-** 96-10-03:	max_arraysize
-** 97-01-29:	Fbstring
-** 97-02-11:	ARIBUFSIZE -> aribufSize in Fstrsplit
-** 97-03-25:	Fstrsplit now PRIVATE
-** 97-03-29:	changed function bytestraddr
-** 97-04-13:	reorg (newintsym)
-** 97-08-01:	removed create_array
-** 00-01-02:    fixed bug in byteswap()
+** 1995-03-05:	Findex changed
+** 1995-03-20:	ARGV sSYSTEMVAR
+** 1995-04-03:	records, pointer
+** 1995-04-07:	alloc
+** 1996-10-03:	max_arraysize
+** 1997-01-29:	Fbstring
+** 1997-02-11:	ARIBUFSIZE -> aribufSize in Fstrsplit
+** 1997-03-25:	Fstrsplit now PRIVATE
+** 1997-03-29:	changed function bytestraddr
+** 1997-04-13:	reorg (newintsym)
+** 1997-08-01:	removed create_array
+** 2000-01-02:  fixed bug in byteswap()
+** 2001-06-17:  corrected type in error message
+** 2002-03-08:  changed direction of sort with user defined sort fcn
+** 2002-03-28:  functions string_scan, stack2string, stack_arraypush, realloc
+** 2002-04-14:  function binsearch
+** 2002-08-05:  small change in GmemBB
+** 2003-02-11:  mkcopy in Fbstring and Ftextstring
+** 2003-06-06:  mkcopy in Srecfield, Farrele and Fstkpush
 */
 
 #include "common.h"
 
+#define QUICKSORT
+
 PUBLIC truc arraysym, stringsym, charsym, stacksym;
 PUBLIC truc vectorsym, pairsym;
 PUBLIC truc mkarrsym, mkstrsym, mkbstrsym;
@@ -61,12 +70,12 @@ PUBLIC truc recordsym, mkrecsym, rec_sym, pointrsym, derefsym;
 
 /**** Prototypen exportierter Funktionen ****/
 PUBLIC void iniarray	_((void));
-PUBLIC void iniargv	_((int argc, char *argv[]));
+PUBLIC void iniargv	    _((int argc, char *argv[]));
 PUBLIC int stringsplit	_((char *str, char *trenn, word2 *offsets));
-PUBLIC int indrange	_((truc *ptr, long len, long *pn0, long *pn1));
+PUBLIC int indrange	    _((truc *ptr, long len, long *pn0, long *pn1));
 PUBLIC truc arrassign	_((truc *arr, truc obj));
 PUBLIC truc subarrassign  _((truc *arr, truc obj));
-PUBLIC void sortarr	_((truc *arr, unsigned len, ifuntt cmpfun));
+PUBLIC void sortarr	    _((truc *arr, unsigned len, ifuntt cmpfun));
 PUBLIC int bytestraddr	_((truc *ptr, truc **ppbstr, byte **ppch,
 						unsigned *plen));
 PUBLIC truc fullrecassign  _((truc *rptr, truc obj));
@@ -77,30 +86,33 @@ PUBLIC truc Pdispose	_((truc *ptr));
 PRIVATE truc argvsym;
 
 PRIVATE truc lengthsym, maxarrsym;
-PRIVATE truc concatsym, splitsym, toupsym, tolowsym, indexsym;
-PRIVATE truc pushsym, popsym, topsym, resetsym, emptysym, stk2arrsym;
+PRIVATE truc concatsym, splitsym, toupsym, tolowsym, indexsym, sscansym;
+PRIVATE truc pushsym, popsym, topsym, resetsym, emptysym;
+PRIVATE truc stk2arrsym, stk2strsym, arrpushsym;
 PRIVATE truc chrsym, ordsym;
-PRIVATE truc sortsym;
+PRIVATE truc sortsym, bsearchsym;
 PRIVATE truc mbtestsym, mbsetsym, mbclrsym, mshiftsym;
 PRIVATE truc mandsym, morsym, mxorsym, mnotsym, mbitswsym, mbyteswsym;
-PRIVATE truc mknewsym, allocsym;
+PRIVATE truc mknewsym, allocsym, reallocsym;
 
 
 /******* Prototypen modul-interner Funktionen ******/
 PRIVATE truc Flength	_((void));
-PRIVATE truc Fchr	_((void));
-PRIVATE truc Ford	_((void));
+PRIVATE truc Fchr	    _((void));
+PRIVATE truc Ford	    _((void));
 PRIVATE truc Fvector	_((int argn));
-PRIVATE truc Fpair	_((void));
+PRIVATE truc Fpair	    _((void));
 PRIVATE truc Fmkarray	_((int argn));
-PRIVATE truc Falloc	_((int argn));
+PRIVATE truc Falloc	    _((int argn));
+PRIVATE truc Srealloc   _((void));
 PRIVATE truc Fmkstring	_((void));
 PRIVATE truc mkstring	_((truc *ptr, int flg, int ch));
 PRIVATE truc Fmkbstring _((void));
 PRIVATE truc Fconcat	_((int argn));
-PRIVATE truc Findex	_((void));
+PRIVATE truc Findex	    _((void));
 PRIVATE long substrindex  _((byte *str, size_t len, byte *sub, size_t len1));
 PRIVATE truc Fstrsplit	_((int argn));
+PRIVATE truc Fstrscan   _((int argn));
 PRIVATE truc Ftolower	_((void));
 PRIVATE truc Ftoupper	_((void));
 PRIVATE truc Gchangecase  _((truc symb));
@@ -112,21 +124,24 @@ PRIVATE truc *vectele	_((truc *ptr, long index));
 PRIVATE int arrcompat	_((int flg1, int flg2));
 PRIVATE long stacklength  _((truc *ptr));
 PRIVATE truc Fstkpush	_((void));
+PRIVATE truc Fstkarrpush  _((int argn));
 PRIVATE truc Fstkpop	_((void));
 PRIVATE truc Fstktop	_((void));
 PRIVATE truc Gstkretr	_((truc symb));
 PRIVATE truc Fstkreset	_((void));
 PRIVATE truc Fstkempty	_((void));
 PRIVATE truc Fstk2array	 _((void));
+PRIVATE truc Fstk2string _((void));
 PRIVATE truc Fmaxarray	 _((void));
 PRIVATE int compfun	_((truc *ptr1, truc *ptr2));
 PRIVATE int ucompfun	_((truc *ptr1, truc *ptr2));
 PRIVATE truc Ssort	_((void));
 PRIVATE truc Hsort1	_((truc *argptr));
 PRIVATE truc Hsort2	_((truc *argptr, truc fun));
+PRIVATE truc Fbsearch   _((int argn));
+PRIVATE truc Hbsearch1  _((truc *ele, truc *vptr, int flg));
 PRIVATE int vectaddr	_((truc *ptr, truc **ppvec, truc **parr,
 					unsigned *plen));
-PRIVATE truc hexstring	_((void));
 PRIVATE truc Ftextstring  _((void));
 PRIVATE truc Fbstring	_((int argn));
 PRIVATE truc Smembtest	_((void));
@@ -193,65 +208,70 @@ PUBLIC void iniarray()
 {
 	truc temp;
 
-	arr_sym	  = newintsym("array[ ]", sFBINARY, (truc)Farrele);
-	subarrsym = newintsym("array[..]",sFBINARY, (truc)Fsubarr);
+	arr_sym	  = newintsym("array[ ]", sFBINARY, (wtruc)Farrele);
+	subarrsym = newintsym("array[..]",sFBINARY, (wtruc)Fsubarr);
 
-	lengthsym = newsymsig("length", sFBINARY, (truc)Flength, s_1);
-	chrsym	  = newsymsig("chr",	sFBINARY, (truc)Fchr, s_1);
-	ordsym	  = newsymsig("ord",	sFBINARY, (truc)Ford, s_1);
-	sortsym	  = newsymsig("sort",	sSBINARY, (truc)Ssort, s_12);
-	concatsym = newsymsig("concat", sFBINARY, (truc)Fconcat, s_1u);
-	splitsym  = newsymsig("string_split",sFBINARY,(truc)Fstrsplit,s_12);
-	toupsym	  = newsymsig("toupper",sFBINARY, (truc)Ftoupper,s_1);
-	tolowsym  = newsymsig("tolower",sFBINARY, (truc)Ftolower,s_1);
-	indexsym  = newsymsig("substr_index",sFBINARY,(truc)Findex,s_2);
+	lengthsym = newsymsig("length", sFBINARY, (wtruc)Flength, s_1);
+	chrsym	  = newsymsig("chr",	sFBINARY, (wtruc)Fchr, s_1);
+	ordsym	  = newsymsig("ord",	sFBINARY, (wtruc)Ford, s_1);
+	sortsym	  = newsymsig("sort",	sSBINARY, (wtruc)Ssort, s_12);
+    bsearchsym= newsymsig("binsearch", sFBINARY, (wtruc)Fbsearch, s_23);
+	concatsym = newsymsig("concat", sFBINARY, (wtruc)Fconcat, s_1u);
+	splitsym  = newsymsig("string_split",sFBINARY,(wtruc)Fstrsplit,s_12);
+    sscansym  = newsymsig("string_scan", sFBINARY,(wtruc)Fstrscan, s_23);
+	toupsym	  = newsymsig("toupper",sFBINARY, (wtruc)Ftoupper,s_1);
+	tolowsym  = newsymsig("tolower",sFBINARY, (wtruc)Ftolower,s_1);
+	indexsym  = newsymsig("substr_index",sFBINARY,(wtruc)Findex,s_2);
 	arraysym  = newsym("array",	sTYPESPEC, nullsym);
 	ofsym	  = newsym("of",	sPARSAUX,  nullsym);
 
 	recordsym = newsym("record",	  sTYPESPEC, nullsym);
-	mkrecsym  = newintsym("record",	  sFBINARY, (truc)Fmkrec0);
-	rec_sym	  = newintsym("rec.field",sSBINARY,(truc)Srecfield);
+	mkrecsym  = newintsym("record",	  sFBINARY, (wtruc)Fmkrec0);
+	rec_sym	  = newintsym("rec.field",sSBINARY,(wtruc)Srecfield);
 	pointrsym = newsym("pointer",	  sTYPESPEC, nil);
-	derefsym  = newintsym("^",	  sSBINARY,(truc)Sderef);
+	derefsym  = newintsym("^",	  sSBINARY,(wtruc)Sderef);
 
 	nullstring= mknullstr();
 	stringsym = newsym("string",   sTYPESPEC, nullstring);
-	str_sym	  = new0symsig("string",sFBINARY,(truc)Ftextstring, s_1);
-	mkstrsym  = newintsym("string[]",sFBINARY, (truc)Fmkstring);
+	str_sym	  = new0symsig("string",sFBINARY,(wtruc)Ftextstring, s_1);
+	mkstrsym  = newintsym("string[]",sFBINARY, (wtruc)Fmkstring);
 
 	nullbstring= mknullbstr();
 	bstringsym= newsym("byte_string",   sTYPESPEC, nullbstring);
-	bstr_sym  = new0symsig("byte_string",sFBINARY,(truc)Fbstring, s_12);
-	mkbstrsym = newintsym("$",  sFBINARY, (truc)Fmkbstring);
+	bstr_sym  = new0symsig("byte_string",sFBINARY,(wtruc)Fbstring, s_12);
+	mkbstrsym = newintsym("$",  sFBINARY, (wtruc)Fmkbstring);
 
-	temp	  = newintsym("",   sSBINARY, (truc)mkstack);
+	temp	  = newintsym("",   sSBINARY, (wtruc)mkstack);
 	stacksym  = newsym("stack", sTYPESPEC, mk0fun(temp));
 
 	charsym	  = newsym("char",  sTYPESPEC, mkchar(' '));
 
-	vectorsym = newintsym("vector", sFBINARY, (truc)Fvector);
-	pairsym	  = newintsym("pair",	sFBINARY, (truc)Fpair);
-	mkarrsym  = newintsym("mkarray",sFBINARY,(truc)Fmkarray);
-	mknewsym  = newsymsig("new",	sSBINARY,(truc)Snew,s_1);
-	allocsym  = newsymsig("alloc",	sFBINARY,(truc)Falloc,s_23);
+	vectorsym = newintsym("vector", sFBINARY, (wtruc)Fvector);
+	pairsym	  = newintsym("pair",	sFBINARY, (wtruc)Fpair);
+	mkarrsym  = newintsym("mkarray",sFBINARY,(wtruc)Fmkarray);
+	mknewsym  = newsymsig("new",	sSBINARY,(wtruc)Snew,s_1);
+	allocsym  = newsymsig("alloc",	sFBINARY,(wtruc)Falloc,s_23);
+    reallocsym= newsymsig("realloc",sSBINARY,(wtruc)Srealloc,s_23);
 
-	pushsym	  = newsymsig("stack_push", sFBINARY,(truc)Fstkpush,s_2);
-	popsym	  = newsymsig("stack_pop",  sFBINARY,(truc)Fstkpop, s_1);
-	topsym	  = newsymsig("stack_top",  sFBINARY,(truc)Fstktop, s_1);
-	resetsym  = newsymsig("stack_reset",sFBINARY,(truc)Fstkreset, s_1);
-	emptysym  = newsymsig("stack_empty",sFBINARY,(truc)Fstkempty, s_1);
-	stk2arrsym= newsymsig("stack2array",sFBINARY,(truc)Fstk2array, s_1);
-	maxarrsym = newsymsig("max_arraysize",sFBINARY,(truc)Fmaxarray, s_0);
-	mbtestsym = newsymsig("mem_btest",  sSBINARY,(truc)Smembtest, s_2);
-	mbsetsym  = newsymsig("mem_bset",   sSBINARY,(truc)Smembset, s_2);
-	mbclrsym  = newsymsig("mem_bclear", sSBINARY,(truc)Smembclear, s_2);
-	mshiftsym = newsymsig("mem_shift",  sSBINARY,(truc)Smemshift, s_2);
-	mandsym	  = newsymsig("mem_and",    sSBINARY,(truc)Smemand, s_2);
-	morsym	  = newsymsig("mem_or",	    sSBINARY,(truc)Smemor, s_2);
-	mxorsym	  = newsymsig("mem_xor",    sSBINARY,(truc)Smemxor, s_2);
-	mnotsym	  = newsymsig("mem_not",    sSBINARY,(truc)Smemnot, s_1);
-	mbitswsym = newsymsig("mem_bitswap",sSBINARY,(truc)Smembitsw, s_1);
-	mbyteswsym= newsymsig("mem_byteswap",sSBINARY,(truc)Smembytesw,s_2);
+	pushsym	  = newsymsig("stack_push", sFBINARY,(wtruc)Fstkpush,s_2);
+	arrpushsym = newsymsig("stack_arraypush", sFBINARY,(wtruc)Fstkarrpush,s_23);
+	popsym	  = newsymsig("stack_pop",  sFBINARY,(wtruc)Fstkpop, s_1);
+	topsym	  = newsymsig("stack_top",  sFBINARY,(wtruc)Fstktop, s_1);
+	resetsym  = newsymsig("stack_reset",sFBINARY,(wtruc)Fstkreset, s_1);
+	emptysym  = newsymsig("stack_empty",sFBINARY,(wtruc)Fstkempty, s_1);
+	stk2arrsym= newsymsig("stack2array",sFBINARY,(wtruc)Fstk2array, s_1);
+	stk2strsym= newsymsig("stack2string",sFBINARY,(wtruc)Fstk2string, s_1);
+	maxarrsym = newsymsig("max_arraysize",sFBINARY,(wtruc)Fmaxarray, s_0);
+	mbtestsym = newsymsig("mem_btest",  sSBINARY,(wtruc)Smembtest, s_2);
+	mbsetsym  = newsymsig("mem_bset",   sSBINARY,(wtruc)Smembset, s_2);
+	mbclrsym  = newsymsig("mem_bclear", sSBINARY,(wtruc)Smembclear, s_2);
+	mshiftsym = newsymsig("mem_shift",  sSBINARY,(wtruc)Smemshift, s_2);
+	mandsym	  = newsymsig("mem_and",    sSBINARY,(wtruc)Smemand, s_2);
+	morsym	  = newsymsig("mem_or",	    sSBINARY,(wtruc)Smemor, s_2);
+	mxorsym	  = newsymsig("mem_xor",    sSBINARY,(wtruc)Smemxor, s_2);
+	mnotsym	  = newsymsig("mem_not",    sSBINARY,(wtruc)Smemnot, s_1);
+	mbitswsym = newsymsig("mem_bitswap",sSBINARY,(wtruc)Smembitsw, s_1);
+	mbyteswsym= newsymsig("mem_byteswap",sSBINARY,(wtruc)Smembytesw,s_2);
 
 	argvsym	  = newsym("ARGV", sSYSTEMVAR, nullsym);
 }
@@ -366,8 +386,7 @@ int argn;	/* argn = 2 or 3 */
 		ch = 0;
 	}
 	else {
-		error(allocsym,
-		"symbol array, string or byte_string expected",*argptr);
+		error(allocsym,err_syarr,*argptr);
 		return(brkerr());
 	}
 	if(argn == 3) {
@@ -387,7 +406,7 @@ int argn;	/* argn = 1 or 2 */
 	truc *argptr, *ptr;
 	truc vector, ele;
 	unsigned i, len;
-	int flg, fixed;
+	int flg;
 
 	argptr = argStkPtr - argn + 1;
 	flg = *FLAGPTR(argptr);
@@ -401,24 +420,19 @@ int argn;	/* argn = 1 or 2 */
 	    return(vector);
 	}
 	WORKpush(vector);
-	if(*FLAGPTR(argStkPtr) & FIXMASK) {
-	    fixed = 1;
-	    ele = *argStkPtr;
-	}
-	else
-	    fixed = 0;
 	if(!len)
 	    len = 1;
-	if(fixed) {
+	if(*FLAGPTR(argStkPtr) >= fBOOL) {
+	    ele = *argStkPtr;
 	    ptr = VECTORPTR(workStkPtr);
 	    while(len--)
-		*ptr++ = ele;
+			*ptr++ = ele;
 	}
 	else {
 	    for(i=0; i<len; i++) {
-		ele = mkarrcopy(argStkPtr);
-		ptr = VECTORPTR(workStkPtr);
-		ptr[i] = ele;
+			ele = mkarrcopy(argStkPtr);
+			ptr = VECTORPTR(workStkPtr);
+			ptr[i] = ele;
 	    }
 	}
 	return(WORKretr());
@@ -460,6 +474,118 @@ int ch;
 	return(string);
 }
 /*--------------------------------------------------------------------*/
+PRIVATE truc Srealloc()
+{
+    truc *argptr, *ptr1, *ptr2;
+	truc res, ele;
+    truc string, vector;
+    char *cpt1, *cpt2;
+    unsigned len, len1;
+	int argn, k;
+	int flg, flg1;
+    int ch;
+
+	argn = *ARGCOUNTPTR(evalStkPtr);
+    argptr = ARG1PTR(evalStkPtr);
+    flg = is_lval(argptr);
+    switch(flg) {
+    case aERROR:
+        error(reallocsym,err_lval,eval(argptr));
+        return brkerr();
+    case vSUBARRAY:
+        error(reallocsym,"subarray cannot be realloc'd",eval(argptr));
+        return brkerr();
+    case vCONST:
+        error(reallocsym,"constants cannot be realloc'd",*argptr);
+        return brkerr();
+    default:
+        break;
+    }
+	res = eval(argptr);
+	ARGpush(res);
+    res = eval(ARGNPTR(evalStkPtr,2));
+    ARGpush(res);
+    if(*FLAGPTR(argStkPtr) != fFIXNUM || *SIGNPTR(argStkPtr)) {
+        error(reallocsym,err_pfix,*argStkPtr);
+        ARGnpop(2);
+        return(brkerr());
+    }
+    len1 = *WORD2PTR(argStkPtr);
+    ARGpop();
+	flg = *FLAGPTR(argStkPtr);
+    if(flg == fVECTOR || flg == fSTRING || flg == fBYTESTRING) {
+        len = *VECLENPTR(argStkPtr);
+    }
+    else {
+        error(reallocsym,err_arr,*argStkPtr);
+        ARGpop();
+        return(brkerr());
+    }
+    if(len1 <= len) {
+        *VECLENPTR(argStkPtr) = len1;
+        res = ARGretr();
+        return Lvalassign(ARG1PTR(evalStkPtr),res);
+    }
+    /* now len1 > len, i.e. proper realloc */
+    if(argn == 3) {
+        res = eval(ARGNPTR(evalStkPtr,3));
+        ARGpush(res);
+    }
+    else if(flg == fVECTOR)
+        ARGpush(zero);
+    if(flg != fVECTOR) {    /* flg == fSTRING || flg == fBYTESTRING */
+        if(argn == 3) {
+            flg1 = *FLAGPTR(argStkPtr);
+            if(flg1 == fCHARACTER || flg1 == fFIXNUM) {
+                ch = *WORD2PTR(argStkPtr);
+			    if(*SIGNPTR(argStkPtr))
+				    ch = -ch;
+            }
+            else {
+                ch = 0;
+            }
+            ARGpop();
+        }
+        else {
+            ch = 0;
+        }
+       	string = (flg == fSTRING ? mkstr0(len1) : mkbstr0(len1));
+    	cpt1 = STRINGPTR(&string);
+        cpt2 = STRINGPTR(argStkPtr);
+        for(k=0; k<len; k++)
+            *cpt1++ = *cpt2++;
+        for(k=len; k<len1; k++)
+            *cpt1++ = ch;
+        *cpt1 = 0;
+        ARGpop();
+        res = Lvalassign(ARG1PTR(evalStkPtr),string);
+        return res;
+    }
+    /* else flg == fVECTOR */
+    vector = mkvect0(len1);
+    WORKpush(vector);
+    ptr1 = VECTORPTR(workStkPtr);
+    ptr2 = VECTORPTR(argStkPtr-1);
+    for(k=0; k<len; k++)
+        *ptr1++ = *ptr2++;
+    if(*FLAGPTR(argStkPtr) >= fBOOL) {
+        ele = *argStkPtr;
+        for(k=len; k<len1; k++)
+            *ptr1++ = ele;
+    }
+    else {
+        for(k=len; k<len1; k++) {
+            ele = mkarrcopy(argStkPtr);
+			ptr1 = VECTORPTR(workStkPtr);
+			ptr1[k] = ele;
+        }
+    }
+    ARGnpop(2);
+    res = Lvalassign(ARG1PTR(evalStkPtr),*workStkPtr);
+    WORKpop();
+    return res;
+}
+/*--------------------------------------------------------------------*/
 PRIVATE truc Fconcat(argn)
 int argn;
 {
@@ -657,6 +783,65 @@ word2 *offsets;
 	return(count);
 }
 /*--------------------------------------------------------------------*/
+/*
+** string_scan(s,bag: string): integer;
+**   returns the position in s of the first character that belongs to bag;
+**   if there is no such character, -1 is returned
+** string_scan(s,bag: string; false): integer;
+**   returns the position in s of the first character that does not
+**   belong to bag; if all characters of s also belog to bag,
+**   -1 is returned
+*/
+PRIVATE truc Fstrscan(argn)
+int argn;
+{
+    truc *argptr;
+    byte *str, *bag, *sep;
+    int flg, tst, ch;
+    int k, len, len2, pos;
+	word4 sep4[64];		/* space for 256 bytes */
+	word4 *pt4;
+    word4 fill;
+
+    if(argn == 3 && (*argStkPtr == zero || *argStkPtr == false)) {
+        fill = 0xFFFFFFFF; tst = 0;
+    }
+    else {
+        fill = 0; tst = 0xFF;
+    }
+    argptr = argStkPtr - argn + 1;
+	for(k=0; k<2; k++) {
+		flg = *FLAGPTR(argptr+k);
+		if(flg != fSTRING && flg != fBYTESTRING) {
+			error(sscansym,err_str,argptr[k]);
+			return(brkerr());
+		}
+	}
+    str = BYTEPTR(argptr);
+    len = *STRLENPTR(argptr);
+	bag = BYTEPTR(argptr+1);
+    len2 = *STRLENPTR(argptr+1);
+
+	k = 64; pt4 = sep4;
+	while(--k >= 0)
+		*pt4++ = fill;
+
+	sep = (byte *)sep4;
+	for(k=0; k<len2; k++) {
+        ch = bag[k];
+		sep[ch] = tst;
+    }
+    pos = -1;
+    for(k=0; k<len; k++) {
+        ch = *str++;
+        if(sep[ch]) {
+            pos = k;
+            break;
+        }
+    }
+    return(mksfixnum(pos));
+}
+/*--------------------------------------------------------------------*/
 PRIVATE truc Ftolower()
 {
 	return(Gchangecase(tolowsym));
@@ -708,7 +893,7 @@ PRIVATE truc Farrele()
 	switch(flg) {
 	case fVECTOR:
 		ptr = vectele(argStkPtr-1,index);
-		if(ptr)
+		if(ptr) 
 			return(*ptr);
 		else
 			return(brkerr());
@@ -1017,7 +1202,7 @@ PRIVATE truc Fstkpush()
 {
 	struct stack *sptr;
 	struct stackpage *spage;
-	truc currpage, vect;
+	truc currpage, sheet;
 	int line;
 
 	if(*FLAGPTR(argStkPtr-1) != fSTACK) {
@@ -1029,16 +1214,16 @@ PRIVATE truc Fstkpush()
 	sptr = (struct stack *)TAddress(argStkPtr-1);
 	line = sptr->line;
 	if(line == 0) {		/* create new page */
-		vect = mkvect0(PAGELEN+1);
+		sheet = mkvect0(PAGELEN+1);
 		sptr = (struct stack *)TAddress(argStkPtr-1);
 		currpage = sptr->page;
-		sptr->page = vect;
-		spage = (struct stackpage *)Taddress(vect);
+		sptr->page = sheet;
+		spage = (struct stackpage *)Taddress(sheet);
 		spage->prevpage = currpage;
 	}
 	else
 		spage = (struct stackpage *)Taddress(sptr->page);
-	spage->data[line] = *argStkPtr;
+	spage->data[line] = mkcopy(argStkPtr); /* 20030606 */
 	line++;
 	if(line >= PAGELEN) {
 		line = 0;
@@ -1048,6 +1233,71 @@ PRIVATE truc Fstkpush()
 	return *argStkPtr;
 }
 /*--------------------------------------------------------------------*/
+/*
+** stack_arraypush(st: stack; vec: array [; direction: integer]): integer;
+*/
+PRIVATE truc Fstkarrpush(argn)
+int argn;
+{
+	struct stack *sptr;
+	struct stackpage *spage;
+	struct vector *vecstruct;
+	truc *argptr, *vec;
+	truc currpage, sheet;
+	int line, incr;
+	unsigned len, pos, k;
+
+	argptr = argStkPtr-argn+1;
+	if(*FLAGPTR(argptr) != fSTACK) {
+		error(arrpushsym,err_stkv,voidsym);
+		return(brkerr());
+	}
+	if(*FLAGPTR(argptr+1) != fVECTOR) {
+		error(arrpushsym,err_vect,argptr[1]);	
+		return(brkerr());
+	}
+	if(argn == 3) {
+		if(*FLAGPTR(argStkPtr) != fFIXNUM) {
+			error(arrpushsym,"integer +1 or -1 expected",*argStkPtr);
+			return(brkerr());
+		}
+		incr = (*SIGNPTR(argStkPtr) ? -1 : 1);
+	}
+	else
+		incr = 1;
+    vecstruct = VECSTRUCTPTR(argptr+1);
+	vec = &(vecstruct->ele0);
+	len = vecstruct->len;
+	if(len == 0)
+		return zero;
+
+	pos = (incr > 0 ? 0 : len-1);
+
+	sptr = (struct stack *)TAddress(argptr);
+	spage = (struct stackpage *)Taddress(sptr->page);
+	line = sptr->line;
+	for(k=0; k<len; k++, pos += incr) {
+		if(line == 0) {		/* create new page */
+			sheet = mkvect0(PAGELEN+1);
+			sptr = (struct stack *)TAddress(argptr);
+			currpage = sptr->page;
+			sptr->page = sheet;
+			spage = (struct stackpage *)Taddress(sheet);
+			spage->prevpage = currpage;
+			vec = VECTORPTR(argptr+1); /* may have changed during gc */
+		}
+		spage->data[line] = vec[pos];
+		line++;
+		if(line >= PAGELEN) {
+			line = 0;
+			sptr->pageno++;
+		}
+	}
+	sptr->line = line;
+
+	return(mkfixnum(len));
+}
+/*--------------------------------------------------------------------*/
 PRIVATE truc Fstkpop()
 {
 	return(Gstkretr(popsym));
@@ -1131,11 +1381,11 @@ PRIVATE truc Fstk2array()
 	    vec[len] = spage->data[lineno];
 	    lineno--;
 	    if(lineno < 0) {
-		lineno += PAGELEN;
-		if(--pageno < 0)
-			break;
-		currpage = spage->prevpage;
-		spage = (struct stackpage *)Taddress(currpage);
+			lineno += PAGELEN;
+			if(--pageno < 0)
+				break;
+			currpage = spage->prevpage;
+			spage = (struct stackpage *)Taddress(currpage);
 	    }
 	}
 	sptr->line = 0;
@@ -1145,6 +1395,81 @@ PRIVATE truc Fstk2array()
 	return(arr);
 }
 /*--------------------------------------------------------------------*/
+PRIVATE truc Fstk2string()
+{
+	struct stack *sptr;
+	struct stackpage *spage;
+	truc currpage;
+	truc *ptr;
+    char *str, *str1, *str2;
+	long llen;
+	unsigned len, mlen, slen, pos, k;
+	int lineno, pageno;
+	int flg;
+
+	if(*FLAGPTR(argStkPtr) != fSTACK) {
+		error(stk2arrsym,err_stkv,voidsym);
+		return(brkerr());
+	}
+	sptr = (struct stack *)TAddress(argStkPtr);
+	pageno = sptr->pageno;
+	lineno = sptr->line;
+	llen = pageno;
+	llen <<= PAGELENBITS;	   /* times PAGELEN */
+	llen += lineno;
+	mlen = (getblocksize()-1) & 0xFFFC;
+	if(llen > (long)mlen) 
+		goto errexit;
+	len = llen;
+	pos = aribufSize*sizeof(word2) & 0xFFFC;
+	if(pos/4 > mlen)
+		pos = 4*mlen;
+    str = (char*)AriBuf;
+	str[pos] = '\0';
+	currpage = sptr->page;
+	spage = (struct stackpage *)Taddress(currpage);
+	lineno = (lineno > 0 ? lineno-1 : PAGELEN-1);
+	while(len > 0) {
+	    len--;
+	    ptr = spage->data + lineno;
+		flg = *FLAGPTR(ptr);
+		if(flg == fCHARACTER) {
+			if(pos > 0) {
+				pos--;
+				str[pos] = *WORD2PTR(ptr);
+			}
+			else
+				goto errexit;
+		}
+		else if(flg == fSTRING) {
+			str2 = STRINGPTR(ptr);
+			slen = *STRLENPTR(ptr);
+			if(pos >= slen) {
+				pos -= slen;
+				for(str1=str+pos, k=0; k<slen; k++)
+					*str1++ = *str2++;
+			}
+			else
+				goto errexit;
+		}
+	    lineno--;
+	    if(lineno < 0) {
+			lineno += PAGELEN;
+			if(--pageno < 0)
+				break;
+			currpage = spage->prevpage;
+			spage = (struct stackpage *)Taddress(currpage);
+	    }
+	}
+	sptr->line = 0;
+	sptr->pageno = 0;
+	sptr->page = nullsym;
+	return(mkstr(str+pos));
+  errexit:
+	error(stk2strsym,err_stkbig,voidsym);
+	return(brkerr());
+}
+/*--------------------------------------------------------------------*/
 PRIVATE truc Fstkreset()
 {
 	struct stack *sptr;
@@ -1180,10 +1505,21 @@ PRIVATE truc Fmaxarray()
 	return(mkfixnum((len-1) & 0xFFFC));
 }
 /*--------------------------------------------------------------------*/
+#ifdef QUICKSORT
+typedef int (*ifunvv)(const void *, const void *);
+PUBLIC void sortarr(arr,len,cmpfn)
+truc *arr;
+unsigned len;
+ifuntt cmpfn;
+{
+    qsort(arr,(size_t)len,sizeof(truc),(ifunvv)cmpfn);
+}
+#else
+/*---------------------------------------------------------*/
 /*
 ** destructively shellsorts array arr[0],...,arr[len-1]
 ** with ordering given by
-**	int cmpfn(truc *ptr1, truc *ptr2);
+**		int cmpfn(truc *ptr1, truc *ptr2);
 ** arr should be safe w.r.t garbage collection in case cmpfn
 ** allocates new memory
 */
@@ -1199,24 +1535,25 @@ ifuntt cmpfn;
 	unsigned i, k, d;
 	int n = 0;
 
-	WORKpush(zero);
+	ARGpush(zero);
 	while(n < DDLEN && dd[n] < len)
 		n++;
 	while(--n >= 0) {
 		d = dd[n];
 		i = len - d;
 		while(i) {
-			*workStkPtr = arr[--i];
+			*argStkPtr = arr[--i];
 			k = i + d;
-			while(k<len && cmpfn(workStkPtr,arr+k) < 0) {
+			while(k<len && cmpfn(argStkPtr,arr+k) > 0) {
 				arr[k-d] = arr[k];
 				k += d;
 			}
-			arr[k-d] = *workStkPtr;
+			arr[k-d] = *argStkPtr;
 		}
 	}
-	WORKpop();
+	ARGpop();
 }
+#endif  /* QUICKSORT */
 /*---------------------------------------------------------*/
 PRIVATE int tttype;
 PRIVATE int compfun(ptr1,ptr2)
@@ -1225,11 +1562,11 @@ truc *ptr1, *ptr2;
 	char *str1, *str2;
 
 	if(tttype >= fFIXNUM)
-		return(cmpnums(ptr2,ptr1,tttype));
+		return(cmpnums(ptr1,ptr2,tttype));
 	else if(tttype == fSTRING) {
 		str1 = STRINGPTR(ptr1);
 		str2 = STRINGPTR(ptr2);
-		return(strcmp(str2,str1));
+		return(strcmp(str1,str2));
 	}
 	else
 		return(0);	 /* this case should not happen */
@@ -1254,7 +1591,7 @@ truc *ptr1, *ptr2;
 		return(*SIGNUMPTR(&res) ? -1 : 1);
 	else {
 		error(sortsym,err_case,mkfixnum(flg));
-		return(0);
+		return(aERROR);
 	}
 }
 /*---------------------------------------------------------*/
@@ -1280,7 +1617,7 @@ PRIVATE truc Ssort()
 		sptr = symptr(obj);
 		if(*FLAGPTR(sptr) != sFUNCTION)
 			goto errexit;
-		fun = sptr->bind;
+		fun = sptr->bind.t;
 		fundefptr = (struct fundef *)Taddress(fun);
 		if(fundefptr->argc != 2)
 			goto errexit;
@@ -1369,6 +1706,129 @@ truc fun;
 	return(eval(argptr));
 }
 /*---------------------------------------------------------*/
+/*
+** binsearch(ele,vec,[compfun]): integer;
+**   Searches for ele in the array vec; 
+**   returns position; if not found, -1 is returned.
+**   The array must be sorted.
+**   The optional argument compfun is a comparing function
+**   as in sort
+*/
+PRIVATE truc Fbsearch(argn)
+int argn;
+{
+   	struct symbol *sptr;
+	truc *argptr, *vptr, *ele;
+	int flg, vergl;
+	unsigned n1, n2, m;
+
+    argptr = argStkPtr - argn + 1;
+	vptr = argptr + 1;
+	flg = *FLAGPTR(vptr);
+	if(flg != fVECTOR) {
+		error(bsearchsym,err_vect,*vptr);
+		return(brkerr());
+	}
+
+    if(argn == 2) {     /* default compare function for numbers or strings */
+        flg = *FLAGPTR(argptr);
+        if(flg >= fFIXNUM || flg == fSTRING) {
+			return Hbsearch1(argptr,vptr,flg);
+        }
+        else {
+            error(bsearchsym,"lacking compare function",voidsym);
+            return(brkerr());
+        }
+    }
+    else {      /* argn=3, user supplied compare function */
+		if(*FLAGPTR(argStkPtr) != fSYMBOL)
+			goto badcompare;
+		sptr = SYMPTR(argStkPtr);
+		if(*FLAGPTR(sptr) != sFUNCTION)
+			goto badcompare;
+		*argStkPtr = sptr->bind.t;
+		if(*FUNARGCPTR(argStkPtr) != 2)
+			goto badcompare;
+	    usercmpfun = argStkPtr;		/* global variable */
+
+		n1 = 0;
+		n2 = *VECLENPTR(vptr);
+		while(n2 > n1) {
+			m = (n1 + n2)/2;
+			ele = VECTORPTR(vptr) + m;
+			vergl = ucompfun(argptr,ele);
+			if(vergl < 0)
+				n2 = m;
+			else if(vergl > 0)
+				n1 = m+1;
+			else
+				return(mkfixnum(m));
+		}
+		return(mksfixnum(-1));
+	}
+  badcompare:
+    error(bsearchsym,"bad compare function",voidsym);
+    return(brkerr());
+}
+/*---------------------------------------------------------*/
+/*
+** binary search in array of numbers or strings
+*/
+PRIVATE truc Hbsearch1(ele,vptr,flg)
+truc *ele, *vptr;
+int flg;
+{
+	int flg1, vergl;
+	truc *arr;
+	unsigned n1, n2, m;
+
+	arr = VECTORPTR(vptr);
+	n1 = 0;
+	n2 = *VECLENPTR(vptr);
+	if(flg == fSTRING) {
+		tttype = flg;
+		while(n2 > n1) {
+			m = (n1 + n2)/2;
+			flg1 = *FLAGPTR(arr+m);
+			if(flg1 != flg) {
+				error(bsearchsym,err_str,arr[m]);
+				return(brkerr());
+			}
+			vergl = compfun(ele,arr+m);
+			if(vergl < 0)
+				n2 = m;
+			else if(vergl > 0)
+				n1 = m+1;
+			else
+				return(mkfixnum(m));
+		}
+		return(mksfixnum(-1));
+	}
+	else if(flg >= fFIXNUM) {
+		while(n2 > n1) {
+			m = (n1 + n2)/2;
+			flg1 = *FLAGPTR(arr+m);
+			if(flg1 < fFIXNUM) {
+				error(bsearchsym,err_num,arr[m]);
+				return(brkerr());
+			}
+			tttype = (flg1 > flg ? flg1 : flg);
+			vergl = compfun(ele,arr+m);
+			if(vergl < 0)
+				n2 = m;
+			else if(vergl > 0)
+				n1 = m+1;
+			else
+				return(mkfixnum(m));
+		}
+		return(mksfixnum(-1));
+	}
+	else {
+		error(bsearchsym,err_case,voidsym);
+		return(brkerr());
+	}
+}
+/*---------------------------------------------------------*/
 PRIVATE int vectaddr(ptr,ppvec,parr,plen)
 truc *ptr;
 truc **ppvec;
@@ -1515,34 +1975,33 @@ truc *pptr;
 	return(ptr);
 }
 /*--------------------------------------------------------------------*/
-PRIVATE truc hexstring()
-{
-	variant v;
-
-	v.xx = *argStkPtr;
-	v.pp.b0 = fBYTESTRING;
-	return(v.xx);
-}
-/*---------------------------------------------------------*/
 PRIVATE truc Ftextstring()
 {
 	variant v;
 	int flg;
 
 	flg = *FLAGPTR(argStkPtr);
-	if(flg != fSTRING && flg != fBYTESTRING) {
+    if(flg == fSTRING) {
+        return(*argStkPtr);
+    }
+	else if(flg != fBYTESTRING) {
 		error(str_sym,err_bystr,*argStkPtr);
 		return(brkerr());
 	}
+    else { /* flg == fBYTESTRING */
+        *argStkPtr = mkcopy(argStkPtr);
+    }
 	v.xx = *argStkPtr;
 	v.pp.b0 = fSTRING;
 	return(v.xx);
 }
 /*---------------------------------------------------------*/
 /*
-** byte_string(x,n)
-** Interpretiert integer x als byte_string der Laenge n;
-** negative Zahlen in Zweier-Komplement-Darstellung
+** byte_string(x,n: integer): byte_string;
+**   Interpretiert integer x als byte_string der Laenge n;
+**   negative Zahlen in Zweier-Komplement-Darstellung
+** byte_string(s: string): byte_string;
+**   verwandelt string in byte_string;
 */
 PRIVATE truc Fbstring(argn)
 int argn;
@@ -1554,13 +2013,23 @@ int argn;
 	unsigned len, i, m;
 	unsigned u, v;
 	int flg, n, sign, pad;
+	variant vv;
 
-	flg = *FLAGPTR(argStkPtr);
-	if(argn == 1 && (flg == fSTRING || flg == fBYTESTRING)) {
-		return(hexstring());
-	}
 	argptr = argStkPtr - argn + 1;
-	if(chkints(bstringsym,argptr,argn) == aERROR) {
+	flg = *FLAGPTR(argptr);
+	if(argn == 1) {
+        if(flg == fSTRING) {
+            *argStkPtr = mkcopy(argStkPtr);
+            vv.xx = *argStkPtr;
+            vv.pp.b0 = fBYTESTRING;
+            return(vv.xx);
+        }
+        else if(flg == fBYTESTRING) {
+            return *argStkPtr;
+        }
+	}
+	if(flg < fINTTYPE0 || flg > fINTTYPE1) {
+        error(bstringsym,err_intt,*argptr);
 		return(brkerr());
 	}
 	if(argn == 2) {
@@ -1575,9 +2044,13 @@ int argn;
 		len = (bit_length(x,n) + 7) >> 3;
 	}
 	bstr = mkbstr0(len);
-	x = AriBuf;
-	n = twocretr(argptr,x);
 	ptr = (byte *)STRING(bstr);
+	x = AriBuf;
+    if(flg == fGF2NINT) {
+        n = bigretr(argptr,x,&sign);
+        pad = 0;
+    }
+	n = twocretr(argptr,x);
 	pad = (n >= (len+1)/2 ? 0 : 1);
 	m = (pad ? n : len/2);
 	if(pad)
@@ -1716,8 +2189,10 @@ truc symb;
 		goto errexit;
 
 	if(symb == mnotsym) {
-		while(len--)
-		    *cpt++ = ~*cpt;
+		while(len--) {
+            u = *cpt;
+		    *cpt++ = ~u;
+        }
 		return(*bstrptr);
 	}
 	else if(symb == mbitswsym) {
@@ -1889,7 +2364,9 @@ PRIVATE truc Srecfield()
 	if(ptr == NULL) {
 		return(brkerr());
 	}
-	return(*ptr);
+	else {
+		return(*ptr);
+	}
 }
 /*--------------------------------------------------------------------*/
 PRIVATE truc Sderef()
@@ -2013,7 +2490,7 @@ truc field;
 		}
 	}
 	if(flg != fRECORD) {
-		error(recordsym,"record variable expexted",*rptr);
+		error(recordsym,"record variable expected",*rptr);
 		return(NULL);
 	}
 	ptr = TAddress(rptr);
