@@ -738,6 +738,7 @@ struct fornode {
 #define FLTEXPOPTR(p)	((int2 *)(bTAddress(p) + OFFSETexpo))
 
 #define ARGCOUNTPTR(p)	((word2 *)(bTAddress(p) + OFFSETargcount))
+#define OPNODEPTR(p)    (truc *)bTAddress(p)
 #define ARG0PTR(p)		((truc *)(bTAddress(p) + OFFSETarg0))
 #define ARG1PTR(p)		((truc *)(bTAddress(p) + OFFSETarg1))
 #define ARGNPTR(p,n)	((truc *)(bTAddress(p) + OFFSETargn(n)))
@@ -1124,7 +1125,7 @@ extern truc unbindsym	_((truc *ptr));
 extern truc unbinduser  _((void));
 
 extern truc	exitsym, exitfun, ret_sym, retsym;
-extern truc     lpbrksym, lpbrkfun, lpcontsym, lpcontfun;
+extern truc lpbrksym, lpbrkfun, lpcontsym, lpcontfun;
 extern truc	equalsym, nequalsym;
 extern truc	funcsym, procsym, beginsym, endsym;
 extern truc	extrnsym, constsym, typesym;
@@ -1241,6 +1242,7 @@ extern truc newsym	_((char *name, int flg, truc bind));
 extern truc newsymsig	_((char *name, int flg, wtruc bind, int sig));
 extern truc new0symsig	_((char *name, int flg, wtruc bind, int sig));
 extern truc mkcopy	_((truc *x));
+extern truc mkcopy0  _((truc *x));
 extern truc mkarrcopy	_((truc *x));
 extern truc mkinum	_((long n));
 extern truc mkarr2	_((unsigned w0, unsigned w1));
