@@ -28,7 +28,7 @@ Address of the author
 Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
-
+#define TYPEIDENT
 /*
 ** control.c
 ** function definition, logical and control functions
@@ -47,6 +47,8 @@ Email	forster@mathematik.uni-muenchen.de
 ** 2002-04-04: simultaneous assignment (x1,...,xn) := (val1,...,valn)
 ** 2002-04-27: gmtime
 ** 2003-02-28: case fGF2NINT in nulltest()
+** 2004-06-20: function type_ident
+** 2004-10-30: for-loop can now do more than 2**32 iterations
 */
 
 #include "common.h"
@@ -116,6 +118,11 @@ PRIVATE truc Smkunbound	  _((void));
 PRIVATE truc Fsymbols	_((void));
 PRIVATE int symbcmp	_((truc *ptr1, truc *ptr2));
 
+#ifdef TYPEIDENT
+PRIVATE truc typeidsym;
+PRIVATE truc Ftypeident _((void));
+PRIVATE int typevalue   _((truc symb));
+#endif
 /*----------------------------------------------------------------------*/
 PUBLIC void inicont()
 {
@@ -192,7 +199,7 @@ PUBLIC void inicont()
     SYMcc1(andsym) = ANDTOK;
 
 	or_sym	   = newintsym("or", sSBINARY, (wtruc)Sor);
-	orsym	   = newsym("or",	sINFIX, or_sym); 
+	orsym	   = newsym("or",	sINFIX, or_sym);
     SYMcc1(orsym) = ORTOK;
 
 	truesym	   = newsym("true",  sSCONSTANT, true);
@@ -211,6 +218,9 @@ PUBLIC void inicont()
 	mkunbdsym  = newsymsig("make_unbound", sSBINARY,
 				(wtruc)Smkunbound, s_bV);
 	symbolssym = newsymsig("symbols", sFBINARY, (wtruc)Fsymbols, s_1);
+#ifdef TYPEIDENT
+	typeidsym  = newsymsig("type_ident", sFBINARY, (wtruc)Ftypeident, s_1);
+#endif
 }
 /*----------------------------------------------------------------------*/
 PRIVATE truc Fequal()
@@ -786,14 +796,14 @@ PUBLIC truc Sfor()
 	struct symbol *sptr;
 	truc *runvar;
 	truc *ptr, *arr;
-	truc *savptr;
+	truc *argptr0, *saveptr0;
 	truc obj;
 	truc res = voidsym;
 
-	word2 *x, *y, *lauf, *inc;
+	word2 *x, *y, *lauf, *inc, *zaehler;
 	word4 anz;
 	int flg;
-	int i, n, n0, m, slen, bodylen, inclen, rlen;
+	int i, n, n0, m, slen, bodylen, inclen, zlen, rlen;
 	int sign, sign1;
 	int toolong = 0;
 
@@ -812,7 +822,8 @@ PUBLIC truc Sfor()
 		error(forsym,err_case,mkfixnum(flg));
 		return(brkerr());
 	}
-	savptr = argStkPtr;
+	argptr0 = argStkPtr;
+    saveptr0 = saveStkPtr;
 	arr = workStkPtr + 1;
 	ARGpush(fptr->inc);
 	ARGpush(fptr->end);
@@ -843,17 +854,28 @@ PUBLIC truc Sfor()
 	else {
 		n = divbig(x,n,y,inclen,AriBuf,&rlen,AriScratch);
 		if(n > 2) {
-			anz = 0xFFFFFFFF;
 			toolong = 1;
 		}
 		else {
 			anz = big2long(AriBuf,n);
-			anz++;
-			if(!anz) {
-				anz = 0xFFFFFFFF;
+			if(anz < 0xFFFFFFFF) {
+                anz++;
+            }
+			else {
 				toolong = 1;
 			}
 		}
+        if(toolong) {
+            zaehler = SAVEspace(n/2+2);
+            if(zaehler) {
+                cpyarr(AriBuf,n,zaehler);
+                zlen = incarr(zaehler,n,1);
+            }
+            else {
+        		error(forsym,err_savstk,voidsym);
+		        goto cleanup;
+            }
+        }
 	}
 	n0 = bigref(argStkPtr,&x,&sign);	/* start */
 	m = (n0 < inclen ? inclen : n0) + 3;
@@ -868,24 +890,37 @@ PUBLIC truc Sfor()
 		error(forsym,err_savstk,voidsym);
 		goto cleanup;
 	}
-	while(anz) {
-		*runvar = mkint(sign,lauf,n0);
-		obj = arreval(arr,bodylen);
-		if((obj == breaksym) && (*brkmodePtr != lpcontsym)) {
-			if(*brkmodePtr != lpbrksym)
-				res = obj;
-			/* else res = voidsym; */
-			break;
-		}
-		n0 = increment(lauf,n0,&sign,inc,inclen,sign1);
-		anz--;
-	}
-	if(toolong && anz==0) {
-		error(forsym,"for-loop truncated",voidsym);
-	}
-	SAVEnpop(slen);
+    if(!toolong) {
+    	while(anz) {
+	    	*runvar = mkint(sign,lauf,n0);
+		    obj = arreval(arr,bodylen);
+    		if((obj == breaksym) && (*brkmodePtr != lpcontsym)) {
+	    		if(*brkmodePtr != lpbrksym)
+		    		res = obj;
+			    /* else res = voidsym; */
+    			break;
+	    	}
+		    n0 = increment(lauf,n0,&sign,inc,inclen,sign1);
+    		anz--;
+	    }
+    }
+    else {
+    	while(zlen) {
+	    	*runvar = mkint(sign,lauf,n0);
+		    obj = arreval(arr,bodylen);
+    		if((obj == breaksym) && (*brkmodePtr != lpcontsym)) {
+	    		if(*brkmodePtr != lpbrksym)
+		    		res = obj;
+			    /* else res = voidsym; */
+    			break;
+	    	}
+		    n0 = increment(lauf,n0,&sign,inc,inclen,sign1);
+    		zlen = decarr(zaehler,zlen,1);
+	    }
+    }
   cleanup:
-	argStkPtr = savptr;
+    saveStkPtr = saveptr0;
+	argStkPtr = argptr0;
 	workStkPtr = arr - 1;
 	return(res);
 }
@@ -1114,4 +1149,152 @@ truc *ptr1, *ptr2;
 {
 	return(strcmp(SYMNAMEPTR(ptr2),SYMNAMEPTR(ptr1)));
 }
+/*----------------------------------------------------------*/
+PRIVATE truc Ftypeident()
+{
+	int flag, flg1, val;
+	truc obj, symb;
+
+	flag = *FLAGPTR(argStkPtr);
+
+	if(flag==1) {
+		flg1 = *SYMFLAGPTR(argStkPtr);
+		switch(flg1) {
+		case sFUNCTION:
+		case sVFUNCTION:
+		case sFBINARY:
+		case sSBINARY:
+			symb = funcsym;
+			break;
+		case sTYPEDEF:
+            *argStkPtr = *SYMBINDPTR(argStkPtr);
+            flag = *FLAGPTR(argStkPtr);
+                goto weiter;
+		case sPARSAUX:
+			symb = *argStkPtr;
+            if(symb == funcsym || symb == procsym)
+                symb = funcsym;
+            else
+                symb = voidsym;
+			break;
+		case sTYPESPEC:
+			symb = *argStkPtr;
+			break;
+		case sSCONSTANT:
+			symb = *argStkPtr;
+			if(symb == nil)
+				symb = pointrsym;		
+			break; 
+		default:
+			symb = voidsym;
+			break;
+		}
+        goto ausgang;
+	}
+  weiter:
+	switch(flag) {
+		case fBOOL:
+			symb = boolsym;
+			break;
+		case fFIXNUM:
+		case fBIGNUM:
+			symb = integsym;
+			break;
+		case fGF2NINT:
+			symb = gf2nintsym;
+			break;
+		case fCHARACTER:
+			symb = charsym;
+			break;
+		case fSTRING:
+			symb = stringsym;
+			break;
+		case fBYTESTRING:
+			symb = bstringsym;
+			break;
+		case fVECTOR:
+			symb = arraysym;
+			break;
+		case fSTREAM:
+			symb = filesym;
+			break;
+		case fSTACK:
+			symb = stacksym;
+			break;
+		case fRECORD:
+			symb = recordsym;
+			break;
+		case fPOINTER:
+			symb = pointrsym;
+			break;
+        case fBUILTIN1:
+            obj = *OPNODEPTR(argStkPtr);
+            if(obj == mkrecsym) {
+                symb = recordsym;
+                break;
+            }
+            /* else fall through */
+		default:
+			if ((flag & fFLTOBJ) == fFLTOBJ)
+				symb = realsym;
+			else
+				symb = errsym;
+			break;
+	}
+  ausgang:
+    val = typevalue(symb);
+	return mkinum(val);
+}
+/*----------------------------------------------------------*/
+PRIVATE int typevalue(symb)
+truc symb;
+{
+    int val;
+
+    if(symb == boolsym) {
+        val = 1;
+    }
+    else if(symb == integsym) {
+        val = 2;
+    }
+    else if(symb == gf2nintsym) {
+        val = 3;
+    }
+    else if(symb == realsym) {
+        val = 4;
+    }
+    else if(symb == charsym) {
+        val = 10;
+    }
+    else if(symb == stringsym) {
+        val = 11;
+    }
+    else if(symb == bstringsym) {
+        val = 12;
+    }
+    else if(symb == arraysym) {
+        val = 20;
+    }
+    else if(symb == recordsym) {
+        val = 21;
+    }
+    else if(symb == pointrsym) {
+        val = 22;
+    }
+    else if(symb == stacksym) {
+        val = 23;
+    }
+    else if(symb == filesym) {
+        val = 30;
+    }
+    else if(symb == funcsym) {
+        val = 40;
+    }
+    else if(symb == voidsym) {
+        val = 0;
+    }
+    else
+        val = -1;
+    return val;
+}
 /*********************************************************************/
