@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 
@@ -34,12 +34,13 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** routines for file i/o
 **
 ** date of last change
-** 95-01-25:	ariExtens
-** 95-03-11:	loadaux
-** 97-01-24:	moved skiptolabel() from scanner.c to file.c
-** 97-02-03:	findarifile
-** 97-04-13:	changed skiptolabel, reorg (newintsym)
-** 98-12-27:    issepdir()
+** 1995-01-25:	ariExtens
+** 1995-03-11:	loadaux
+** 1997-01-24:	moved skiptolabel() from scanner.c to file.c
+** 1997-02-03:	findarifile
+** 1997-04-13:	changed skiptolabel, reorg (newintsym)
+** 1998-12-27:  issepdir()
+** 2001-06-02:	flush(transcript)
 */
 
 #include "common.h"
@@ -63,6 +64,7 @@ PUBLIC char *ariExtens = ".ari";
 
 PRIVATE truc stdinsym, stdoutsym, stderrsym;
 PRIVATE truc loadsym, openrsym, openwsym, openasym, closesym, flushsym;
+PRIVATE truc getcwdsym, setcwdsym;
 PRIVATE truc binarysym;
 PRIVATE truc rewindsym, setposym, getposym;
 PRIVATE truc rdbytesym, wrbytesym, rdblksym, wrblksym;
@@ -76,6 +78,8 @@ PRIVATE truc F1flush	_((int argn));
 PRIVATE truc Frewind	_((void));
 PRIVATE truc Fgetpos	_((void));
 PRIVATE truc Fsetpos	_((void));
+PRIVATE truc Fgetcwd    _((void));
+PRIVATE truc Fsetcwd    _((void));
 PRIVATE truc Sclose	_((void));
 PRIVATE int Sopen0	_((truc symb, int mode));
 PRIVATE truc Sopenread	  _((void));
@@ -86,13 +90,16 @@ PRIVATE truc Fwrbyte	_((void));
 PRIVATE truc Srdblock	_((void));
 PRIVATE int Gblockaux	_((truc symb,FILE **pfil, size_t *anz, byte **pbuf));
 PRIVATE truc Swrblock	_((void));
+#ifdef genUNiX
+PRIVATE void expandtilde    _((char *buf));
+#endif
 
 /*----------------------------------------------------------------------*/
 PUBLIC void inifile()
 {
 	truc temp;
 
-	temp	  = newintsym("",  sSBINARY, (truc)Sdumstream);
+	temp	  = newintsym("",  sSBINARY, (wtruc)Sdumstream);
 	filesym	  = newsym("file", sTYPESPEC, mk0fun(temp));
 	eofsym	  = newselfsym("eof",sINTERNAL);
 	binarysym = newreflsym("binary", sSYSSYMBOL);
@@ -105,20 +112,22 @@ PUBLIC void inifile()
 	stdoutsym = newsym("stdout", sSCONSTANT, tstdout);
 	stderrsym = newsym("stderr", sSCONSTANT, tstderr);
 
-	loadsym	  = newsymsig("load",	    sFBINARY,(truc)Fload, s_12);
-	openrsym  = newsymsig("open_read",  sSBINARY,(truc)Sopenread, s_23);
-	openwsym  = newsymsig("open_write", sSBINARY,(truc)Sopenwrite, s_23);
-	openasym  = newsymsig("open_append",sSBINARY,(truc)Sopenapp, s_23);
-	closesym  = newsymsig("close",	    sSBINARY,(truc)Sclose, s_bV);
-	flushsym  = newsymsig("flush",	    sFBINARY,(truc)F1flush, s_01);
-	rewindsym = newsymsig("rewind",	    sFBINARY,(truc)Frewind, s_bV);
-	setposym  = newsymsig("set_filepos",sFBINARY,(truc)Fsetpos, s_2);
-	getposym  = newsymsig("get_filepos",sFBINARY,(truc)Fgetpos, s_1);
+	loadsym	  = newsymsig("load",	    sFBINARY,(wtruc)Fload, s_12);
+	openrsym  = newsymsig("open_read",  sSBINARY,(wtruc)Sopenread, s_23);
+	openwsym  = newsymsig("open_write", sSBINARY,(wtruc)Sopenwrite, s_23);
+	openasym  = newsymsig("open_append",sSBINARY,(wtruc)Sopenapp, s_23);
+	closesym  = newsymsig("close",	    sSBINARY,(wtruc)Sclose, s_bV);
+	flushsym  = newsymsig("flush",	    sFBINARY,(wtruc)F1flush, s_01);
+	rewindsym = newsymsig("rewind",	    sFBINARY,(wtruc)Frewind, s_bV);
+	setposym  = newsymsig("set_filepos",sFBINARY,(wtruc)Fsetpos, s_2);
+	getposym  = newsymsig("get_filepos",sFBINARY,(wtruc)Fgetpos, s_1);
+    getcwdsym = newsymsig("get_workdir",sFBINARY,(wtruc)Fgetcwd, s_0);
+    setcwdsym = newsymsig("set_workdir",sFBINARY,(wtruc)Fsetcwd, s_1);
 
-	rdbytesym = newsymsig("read_byte",  sFBINARY,(truc)Frdbyte, s_1);
-	wrbytesym = newsymsig("write_byte", sFBINARY,(truc)Fwrbyte, s_2);
-	rdblksym  = newsymsig("read_block", sSBINARY,(truc)Srdblock, s_3);
-	wrblksym  = newsymsig("write_block",sSBINARY,(truc)Swrblock, s_3);
+	rdbytesym = newsymsig("read_byte",  sFBINARY,(wtruc)Frdbyte, s_1);
+	wrbytesym = newsymsig("write_byte", sFBINARY,(wtruc)Fwrbyte, s_2);
+	rdblksym  = newsymsig("read_block", sSBINARY,(wtruc)Srdblock, s_3);
+	wrblksym  = newsymsig("write_block",sSBINARY,(wtruc)Swrblock, s_3);
 }
 /*----------------------------------------------------------------------*/
 /*
@@ -165,7 +174,7 @@ int argn;		/* argn = 1 or 2 */
 	findarifile(argv[0],name);
 
 	if(verbose) {
-		s1form(OutBuf,"(** loading file ~A **)",(word4)name);
+		s1form(OutBuf,"(** loading file ~A **)",(wtruc)name);
 		fprintline(tstdout,OutBuf);
 	}
 
@@ -179,13 +188,13 @@ int argn;		/* argn = 1 or 2 */
 PUBLIC int issepdir(ch)
 int ch;
 {
-        int i;
+    int i;
 
-        for(i=strlen(SEP_DIR)-1; i>=0; --i) {
-                if(ch == SEP_DIR[i])
-                        return 1;
-        }
-        return 0;
+    for(i=strlen(SEP_DIR)-1; i>=0; --i) {
+        if(ch == SEP_DIR[i])
+            return 1;
+   }
+   return 0;
 }
 /*----------------------------------------------------------------------*/
 /*
@@ -233,11 +242,7 @@ char *paths, *fnam, *buf;
 	int ch;
 	char *ptr, *ptr2;
 	FILE *fil;
-#ifdef genUNiX
-	char hbuf[MAXPFADLEN+2];
-	char *home;
-	int n;
-#endif
+
 	ptr = paths;
 	ch = *ptr;
 	while(ch) {
@@ -251,14 +256,7 @@ char *paths, *fnam, *buf;
 		strcopy(ptr2,fnam);
 #ifdef genUNiX
 		if(strncmp(buf,"~/",2) == 0) {
-		    home = getenv("HOME");
-		    if(home != NULL) {
-			n = strncopy(hbuf,home,MAXPFADLEN);
-			if(issepdir(hbuf[n-1])) 
-				n--;
-			strncopy(hbuf+n,buf+1,MAXPFADLEN-n);
-			strncopy(buf,hbuf,MAXPFADLEN);
-		    }
+            expandtilde(buf);
 		}
 #endif
 		fil = fopen(buf,"r");
@@ -270,6 +268,26 @@ char *paths, *fnam, *buf;
 	return(0);
 }
 /*-------------------------------------------------------------*/
+#ifdef genUNiX
+PRIVATE void expandtilde(buf)
+char *buf;
+{
+	char hbuf[MAXPFADLEN+2];
+	char *home;
+	int n;
+
+    home = getenv("HOME");
+    if(home != NULL) {
+		n = strncopy(hbuf,home,MAXPFADLEN);
+   		if(issepdir(hbuf[n-1]))
+    		n--;
+	    strncopy(hbuf+n,buf+1,MAXPFADLEN-n);
+		strncopy(buf,hbuf,MAXPFADLEN);
+    }
+    return;
+}
+#endif
+/*-------------------------------------------------------------*/
 PUBLIC int fnextens(str,name,extens)
 char *str, *name, *extens;
 {
@@ -440,8 +458,9 @@ int argn;
 	struct stream *strm;
 	FILE *fptr;
 
-	if(argn == 0)
+	if(argn == 0) {
 		fptr = stdout;
+	}      
 	else if(*FLAGPTR(argStkPtr) == fSTREAM) {
 		strm = STREAMPTR(argStkPtr);
 		if(strm->mode & OUTSTREAM)
@@ -449,8 +468,11 @@ int argn;
 		else
 			goto ausgang;
 	}
-	else
+	else {
+		if(*argStkPtr == transcsym)
+			flushlog();
 		goto ausgang;
+      }
 	fflush(fptr);
   ausgang:
 	return(voidsym);
@@ -760,4 +782,42 @@ PRIVATE truc Swrblock()
 	n = fwrite(bpt,1,anz,fil);
 	return(mkfixnum(n));
 }
+/*----------------------------------------------------------*/
+PRIVATE truc Fgetcwd()
+{
+    char *pfad;
+
+    pfad = getworkdir();
+
+    return mkstr(pfad);
+}
+/*----------------------------------------------------------*/
+PRIVATE truc Fsetcwd()
+{
+    int res, len;
+    char pfad[MAXPFADLEN];
+    char *pf1;
+
+    if(*FLAGPTR(argStkPtr) != fSTRING) {
+        error(setcwdsym,err_str,*argStkPtr);
+        return brkerr();
+    }
+    len = *STRLENPTR(argStkPtr);
+    if(len < MAXPFADLEN) {
+        strncopy(pfad,STRINGPTR(argStkPtr),MAXPFADLEN);
+#ifdef genUNiX
+		if(strncmp(pfad,"~/",2) == 0) {
+            expandtilde(pfad);
+        }
+#endif
+        res = setworkdir(pfad);
+    }
+    else
+        res = 0;
+    if(res == 0)
+        pf1 = "";
+    else
+        pf1 = getworkdir();
+    return mkstr(pf1);
+}
 /**************************************************************/
