@@ -2,7 +2,7 @@
 /* file parser.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2004 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -44,6 +44,8 @@ Email	forster@mathematik.uni-muenchen.de
 ** 2002-02-16   uminsym in negate
 ** 2002-02-24   Z1TOK
 ** 2003-02-27   enddotsym
+** 2004-06-13	better error handling in readwhile, readif; ARGLIST
+** 2004-08-27   removed bug in previous handling of ARGLIST
 */
 
 #include "common.h"
@@ -139,6 +141,7 @@ PUBLIC void iniparse()
 #define EXTERNDECL	0x04
 #define POINTRECURS	0x08
 #define PLAINSYM	0x10
+#define ARGLIST     0x20
 #define ALLFLAG	    0xFFFF
 
 PRIVATE int pstat(mode,flag)
@@ -471,7 +474,13 @@ int *cflgptr;
 				*cflgptr = 1;
 			}
 			else if(obj == funcsym || obj == procsym) {
-				obj = compfunc(strom);
+				if (pstat(PQUERY,ARGLIST) == ARGLIST) {
+					obj = funcsym;
+                    nexttok(strom,0);
+				}
+				else {
+					obj = compfunc(strom);
+				}
 				*cflgptr = 1;
 			}
 			else if(obj == varsym) {
@@ -1160,7 +1169,7 @@ truc *strom;
 /*
 ** Die formalen Parameter einer Funktion (Liste symbs) werden mit
 ** fortlaufenden Nummern markiert. Aus der Liste kind ist
-** abzulesen, ob es sich um einen Variablen-Paramenter handelt.
+** abzulesen, ob es sich um einen Variablen-Parameter handelt.
 ** In diesem Fall wird die Nummer um mGLOBAL erhoeht
 */
 PRIVATE int markparms(symbs,kind)
@@ -2003,15 +2012,19 @@ truc *strom;
 int endtok;
 {
 	truc obj;
-	int tok;
+	int tok, ret;
+	int fehler = 0;
 	int n = 0;
     char *errmess;
 
 	tok = nexttok(strom,1);		/* ueberlese linke Klammer */
+	pstat(PSET,ARGLIST);
 	while(tok != endtok) {
 		obj = readexpr(strom,0);
-		if(obj == parserrsym)
-			goto errexit;
+		if(obj == parserrsym) {
+			fehler = 1;
+			break;
+		}
 		else {
 			PARSpush(obj);
 			n++;
@@ -2022,14 +2035,20 @@ int endtok;
 		else if(tok != endtok) {
             errmess = (endtok == RBRACETOK ? err_0brace : err_0rparen);
 			rerror(parserrsym,errmess,voidsym);
-			goto errexit;
+			fehler = 1;
+			break;
 		}
 	}
-	nexttok(strom,0);
-	return(n);
-  errexit:
-	PARSnpop(n);
-	return(aERROR);
+	pstat(PCLEAR,ARGLIST);
+	if(fehler) {
+		PARSnpop(n);
+		ret = aERROR;
+	}
+	else {
+		nexttok(strom,0);
+		ret = n;
+	}
+	return(ret);
 }
 /*--------------------------------------------------------------*/
 /*
@@ -2166,6 +2185,9 @@ truc *strom;
 	savptr = argStkPtr;
 	nexttok(strom,0);	/* ueberspringe while */
 	obj = readexpr(strom,0);
+	if(obj == parserrsym) {
+		goto cleanup;
+	}
 	PARSpush(obj);
 	n++;
 	sflg = obligsym(whilesym,strom,dosym);
@@ -2361,6 +2383,9 @@ truc *strom;
 	nexttok(strom,0);	/* ueberspringe if */
 	while(1) {
 		obj = readexpr(strom,0);
+		if(obj == parserrsym) {
+			goto cleanup;
+		}
 		PARSpush(obj);
 		n++;
 		sflg = obligsym(ifsym,strom,thensym);
@@ -2559,7 +2584,7 @@ int tok;
         name = "integer";
         break;
     case GF2NTOK:
-        name = "gf2n_int";
+        name = "gf2nint";
         break;
     case FLOATTOK:
         name = "floating point number";
