@@ -48,6 +48,12 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** 1997-12-20: small change in fprintline
 ** 1998-10-01: adjustments for Win32GUI
 ** 2001-01-01: multiprec floats
+** 2001-03-30: Win32GUI replaces genWinGUI
+** 2001-06-02: function flushlog
+** 2002-04-01: allow interrupt while printing vectors
+** 2002-08-03: baseprefix changed,
+**      changed argument type of s1form, s2form, sformaux
+** 2003-02-21: Gformat also for gf2nint's
 */
 
 #include "common.h"
@@ -60,6 +66,7 @@ PUBLIC void iniprint	_((int cols));
 PUBLIC int logout	_((int ch));
 PUBLIC void strlogout	_((char *str));
 PUBLIC void closelog	_((void));
+PUBLIC void flushlog	_((void));
 PUBLIC void tprint	_((truc strom, truc obj));
 PUBLIC int strcopy	_((char *tostr, char *fromstr));
 PUBLIC int strncopy	_((char *tostr, char *fromstr, int maxlen));
@@ -68,10 +75,11 @@ PUBLIC void fprintline	_((truc strom, char *str));
 PUBLIC void fnewline	_((truc strom));
 PUBLIC void ffreshline	_((truc strom));
 PUBLIC void flinepos0	_((truc strom));
-PUBLIC int s1form	_((char *buf, char *fmt, word4 dat));
-PUBLIC int s2form	_((char *buf, char *fmt, word4 dat1, word4 dat2));
+PUBLIC int s1form	_((char *buf, char *fmt, wtruc dat));
+PUBLIC int s2form	_((char *buf, char *fmt, wtruc dat1, wtruc dat2));
 PUBLIC int setprnprec _((int prec));
-PUBLIC	truc writesym, writlnsym, formatsym;
+PUBLIC truc writesym, writlnsym, formatsym;
+PUBLIC truc transcsym;
 
 PUBLIC char OutBuf[IOBUFSIZE+4];
 PUBLIC int Log_on = 0;
@@ -103,7 +111,7 @@ PRIVATE truc Fsetpbase	_((void));
 PRIVATE truc Fgetpbase	_((void));
 PRIVATE int admissbase	_((int base));
 PRIVATE int printbase	_((int base));
-PRIVATE char *baseprefix   _((int base));
+PRIVATE char *baseprefix   _((int base, int mode));
 PRIVATE int nibasci0	   _((word2 *x, int k));
 PRIVATE void printfint	   _((truc strom, truc obj, forminfo *fptr));
 PRIVATE void printfloat	   _((truc strom, truc obj, forminfo *fptr));
@@ -130,12 +138,11 @@ PRIVATE int fprintch	_((truc strom, int ch));
 PRIVATE int long2alfa	_((char *buf, long u));
 PRIVATE int long2s0alfa	 _((char *buf, long u, int len));
 PRIVATE int word4xalfa	_((char *buf, word4 u));
-PRIVATE int sformaux	_((forminfo *fmptr, char *buf, word4 dat));
+PRIVATE int sformaux	_((forminfo *fmptr, char *buf, wtruc dat));
 PRIVATE char *formscan	_((char *str, forminfo *fmptr));
 PRIVATE int isformdir	_((int ch));
 
 PRIVATE truc basesym, groupsym, columsym, digssym;
-PRIVATE truc transcsym;
 PRIVATE truc setpbsym, getpbsym;
 PRIVATE truc itoasym, ftoasym, ecvtsym;
 
@@ -163,9 +170,9 @@ int cols;
 	PrintCols = MaxCols-1;
 	PrintPrec = deffltprec();
 
-	write_sym = new0symsig("write",sFBINARY,(truc)F1write, s_1u);
+	write_sym = new0symsig("write",sFBINARY,(wtruc)F1write, s_1u);
 	writesym  = newsym("write",   sPARSAUX, write_sym);
-	writln_sym= new0symsig("writeln",sFBINARY,(truc)Fwritln,s_0u);
+	writln_sym= new0symsig("writeln",sFBINARY,(wtruc)Fwritln,s_0u);
 	writlnsym = newsym("writeln",	sPARSAUX, writln_sym);
 
 	basesym	  = newsym("base",	sUNBOUND, nullsym);
@@ -173,13 +180,13 @@ int cols;
 	columsym  = newsym("columns",	sUNBOUND, nullsym);
 	digssym	  = newsym("digits",	sUNBOUND, nullsym);
 
-	formatsym = newintsym("", sFBINARY, (truc)Gformat);
-	setpbsym  = newsymsig("set_printbase",sFBINARY,(truc)Fsetpbase,s_ii);
-	getpbsym  = newsymsig("get_printbase",sFBINARY,(truc)Fgetpbase,s_0);
-	transcsym = newsymsig("transcript", sFBINARY,(truc)Ftranscript, s_01);
-	itoasym	  = newsymsig("itoa",	   sFBINARY, (truc)Fint2str, s_12);
-	ftoasym	  = newsymsig("ftoa",	   sFBINARY, (truc)Fflt2str, s_1);
-	ecvtsym	  = newsymsig("float_ecvt",sSBINARY,(truc)Secvt, s_4);
+	formatsym = newintsym("", sFBINARY, (wtruc)Gformat);
+	setpbsym  = newsymsig("set_printbase",sFBINARY,(wtruc)Fsetpbase,s_ii);
+	getpbsym  = newsymsig("get_printbase",sFBINARY,(wtruc)Fgetpbase,s_0);
+	transcsym = newsymsig("transcript", sFBINARY,(wtruc)Ftranscript, s_01);
+	itoasym	  = newsymsig("itoa",	   sFBINARY, (wtruc)Fint2str, s_12);
+	ftoasym	  = newsymsig("ftoa",	   sFBINARY, (wtruc)Fflt2str, s_1);
+	ecvtsym	  = newsymsig("float_ecvt",sSBINARY,(wtruc)Secvt, s_4);
 }
 /*------------------------------------------------------------------*/
 PRIVATE ifun putcfun(strom)
@@ -189,7 +196,7 @@ truc strom;
 	if(Log_on && (strom == tstdout || strom == tstderr))
 		return(log2out);
 	else {
-#ifdef Win32GUI
+#ifdef genWinGUI
         if ((strom == tstdout || strom == tstderr))
             return(wincharout);
         else
@@ -253,7 +260,7 @@ PRIVATE int log2out(ch)
 int ch;
 {
 	putc(ch,logfile);
-#ifdef Win32GUI
+#ifdef genWinGUI
     return wincharout(ch);
 #else
 	return putc(ch,stdout);
@@ -310,6 +317,12 @@ int argn;
 		return(true);
 	}
 }
+/*-------------------------------------------------------------------*/
+PUBLIC void flushlog()
+{
+	if(Log_on)
+		fflush(logfile);
+}
 /*------------------------------------------------------------------*/
 PUBLIC void closelog()
 {
@@ -503,7 +516,7 @@ PRIVATE truc Gformat()
 			}
 		}
 	}
-	else if(flg == fFIXNUM || flg == fBIGNUM || flg == fBYTESTRING) {
+	else if((flg >= fINTTYPE0 && flg <= fINTTYPE1) || flg == fBYTESTRING) {
 	    if(m < 4)
 		    for(i=m; i<4; i++)
 			    WORKpush(zero);
@@ -671,6 +684,10 @@ truc strom, obj;
 		printfint(strom,obj,&fmt);
 		return;
 	}
+	else if(flg == fGF2NINT) {
+		printfint(strom,obj,&fmt);
+		return;
+	}
 	else if(flg >= fFLTOBJ) {
 		printfloat(strom,obj,&fmt);
 		return;
@@ -748,15 +765,22 @@ int n;
 			/* Stellen nach dem Dezimalpunkt */
 		}
 	}
-	else if(flg == fFIXNUM || flg == fBIGNUM) {
+	else if(flg == fFIXNUM || flg == fBIGNUM || flg == fGF2NINT) {
 		if(n <= 1)
 			base = printbase(0);
 		else if(n >= 2) {
 			base = *WORD2PTR(arr+1);
 			base = admissbase(base);
 		}
+        if(flg == fGF2NINT) {
+            if(base == 10)
+                base = 16;
+        }
 		if(n < 3 || arr[2] == nullsym) {
-		    len = (flg == fBIGNUM ? VEClen(obj) : 1);
+            if(flg == fBIGNUM || flg == fGF2NINT)
+                len = VEClen(obj);
+            else
+		        len = 1;
 		    switch(base) {
 		    case 16:
 			    group = (len > 2 ? 4 : 8);
@@ -872,19 +896,27 @@ int prec;
 	return (PrintPrec = prec);
 }
 /*--------------------------------------------------------------*/
-PRIVATE char *baseprefix(base)
-int base;
+PRIVATE char *baseprefix(base, mode)
+int base, mode;
 {
+    static char pref[4];
+
 	switch(base) {
 	case 16:
-		return("0x");
+		strcopy(pref,"0x");
+        break;
 	case 8:
-		return("0o");
+		strcopy(pref,"0o");
+        break;
 	case 2:
-		return("0y");
+		strcopy(pref,"0y");
+        break;
 	default:
-		return("");
+		pref[0] = '\0';
 	}
+    if(mode == 2 && strlen(pref) > 1)
+        pref[0] = '2';
+    return pref;
 }
 /*--------------------------------------------------------------*/
 /*
@@ -902,6 +934,7 @@ forminfo *fptr;
 	int k, n, diff, bpd;
 	int dig;
 	int rightpad;
+    int mode;
 
 	big = obj;
 	len = bigretr(&big,AriBuf,&sign);
@@ -928,8 +961,9 @@ forminfo *fptr;
 	if(len == 0 && noofdigs == 0)
 		noofdigs = 1;
 
-	if(base >= 10)
+	if(base >= 10) {
 		bpd = 4;
+    }
 	else if(base == 8) {
 		x[len] = 0;	/* !!! */
 		bpd = 3;
@@ -943,8 +977,10 @@ forminfo *fptr;
 	pp = (NN + grp - 1)/grp;  /* Zahl der Bloecke */
 	k = NN - (pp-1)*grp;	  /* Zahl der Ziffern im 1.Block */
 	n = (sign ? strcopy(OutBuf,"-") : 0);
-	if(quotemode)
-		n += strcopy(OutBuf+n,baseprefix(base));
+	if(quotemode) {
+        mode = (FLAG(big) == fGF2NINT ? 2 : 0);
+		n += strcopy(OutBuf+n,baseprefix(base,mode));
+    }
 	m = NN + n + (grp==1 ? 0 : (pp-1));
 
 	rightpad = 0;
@@ -1006,7 +1042,6 @@ forminfo *fptr;
 	int k, len, digs, dec, width, sign, mode;
 	int eflg;
 	int grp=5;
-	char *out;
 
 	mode = fptr->mode;
 	width = fptr->param[0];
@@ -1072,7 +1107,7 @@ forminfo *fptr;
 		fprintmarg(strom,OutBuf,strlen(OutBuf));
 	}
 	if(eflg) {
-		k = s1form(OutBuf,"e~D",expo);
+		k = s1form(OutBuf,"e~D",(wtruc)expo);
 		fprintmarg(strom,OutBuf,k);
 	}
 }
@@ -1126,6 +1161,12 @@ int flg;
 			if(pos >= 1 && pos <= PrintCols - 1)
 				fprintch(strom,' ');
 		}
+        if((n & 0xF) == 0 && INTERRUPT) {
+            setinterrupt(0);
+            len = strcopy(OutBuf,"...user interrupt... ");
+            fprintmarg(strom,OutBuf,len);
+            break;
+        }
 		ptr++;
 	}
 	WORKpop();
@@ -1316,7 +1357,6 @@ word2 *hilf;
 	word2 *scratch;
 	int len,len1,n,fill;
 	int mode, width, wd1, prec, prec1, dec;
-	int expch;
 
 	acc.digits = hilf;
 
@@ -1379,7 +1419,7 @@ word2 *hilf;
 			expo = acc.expo + prec - 1;
 			acc.expo = -prec + 1;
 			n = fixstring(&acc,prec-1,buf);
-			n += s1form(buf+n,"e~D",expo);
+			n += s1form(buf+n,"e~D",(wtruc)expo);
 		}
 	}
 	else {
@@ -1443,10 +1483,10 @@ forminfo *fptr;
 	ch = v.pp.ww;
 	if(quotemode) {
 		if(ch < ' ' || ch == 127) {
-			len = s1form(buf,"chr(~D)",(word4)ch);
+			len = s1form(buf,"chr(~D)",(wtruc)ch);
 		}
 		else {
-			len = s1form(buf,"'~C'",(word4)ch);
+			len = s1form(buf,"'~C'",(wtruc)ch);
 		}
 		return(len);
 	}
@@ -1495,7 +1535,7 @@ char *buf;
 	obj = PTRtarget(obj);
 	if(obj == nil)
 		return(strcopy(buf,SYMname(nil)));
-	return(s1form(buf,"<PTR^~X>",truc2msf(obj)));
+	return(s1form(buf,"<PTR^~X>",(wtruc)truc2msf(obj)));
 }
 /*--------------------------------------------------------------*/
 PRIVATE int obj2str(flg,obj,buf)
@@ -1525,7 +1565,7 @@ char *buf;
 		else
 			str = "OBJECT";
 	}
-	return(s2form(buf,"<~A:~X>",(word4)str,u));
+	return(s2form(buf,"<~A:~X>",(wtruc)str,(wtruc)u));
 }
 /*--------------------------------------------------------------*/
 PRIVATE int leftpad(buf,width,ch)
@@ -1724,19 +1764,19 @@ word4 u;
 PUBLIC int s1form(buf,fmt,dat)
 char *buf;
 char *fmt;
-word4 dat;
+wtruc dat;
 {
-	return(s2form(buf,fmt,dat,(word4)0));
+	return(s2form(buf,fmt,dat,(wtruc)0));
 }
 /*--------------------------------------------------------------*/
 PUBLIC int s2form(buf,fmt,dat1,dat2)
 char *buf;
 char *fmt;
-word4 dat1, dat2;
+wtruc dat1, dat2;
 {
 	forminfo finf;
 	char *fmt1;
-	word4 dat;
+	wtruc dat;
 	int mode;
 	int n = 0;
 	int count = 0;
@@ -1772,12 +1812,14 @@ word4 dat1, dat2;
 PRIVATE int sformaux(fmptr,buf,dat)
 forminfo *fmptr;
 char *buf;
-word4 dat;
+wtruc dat;
 {
 	char *ptr1, *ptr2;
-	int i, len, width;
+	int i, len, width, mode;
+    char fill;
 
-	switch(fmptr->mode) {
+    mode = fmptr->mode;
+	switch(mode) {
 		case 'A':
 			ptr1 = (char *)dat;
 			len = strcopy(buf,ptr1);
@@ -1786,10 +1828,10 @@ word4 dat;
 			len = long2alfa(buf,(long)dat);
 			break;
 		case 'X':
-			len = word4xalfa(buf,dat);
+			len = word4xalfa(buf,(word4)dat);
 			break;
 		case 'C':
-			buf[0] = dat;
+			buf[0] = (char)dat;
 			buf[1] = 0;
 			len = 1;
 			break;
@@ -1799,12 +1841,13 @@ word4 dat;
 	}
 	width = fmptr->param[0];
 	if(width > len) {
+        fill = fmptr->param[1];
 		ptr2 = buf + width;
 		ptr1 = buf + len;
 		for(i=0; i<=len; i++, ptr1--, ptr2--)
 			*ptr2 = *ptr1;
 		for(i=len; i<width; i++, ptr2--)
-			*ptr2 = ' ';
+			*ptr2 = fill;
 		len = width;
 	}
 	return(len);
@@ -1814,7 +1857,7 @@ PRIVATE char *formscan(str,fmptr)
 char *str;
 forminfo *fmptr;
 {
-	int k, n, x;
+	int n, x;
 	int sign, ch;
 
 	if(*str == 0)
@@ -1829,24 +1872,20 @@ forminfo *fmptr;
 		fmptr->param[0] = n;
 		return(str);
 	}
-	for(k=0; k<FORMPARAM; k++) {
+
+	str++;
+	ch = *str;
+	if(ch == '0') {
+        fmptr->param[1] = '0';
 		str++;
-		sign = 0;
-		ch = str[0];
-		if(ch == '+')
-			str++;
-		else if(ch == '-') {
-			sign = -1;
-			str++;
-		}
-		x = str2int(str,&n);
-		if(n > 0) {
-			fmptr->param[k] = (sign ? -x : x);
-			str += n;
-		}
-		if(*str != ',')
-			break;
-	}
+    }
+    else {
+        fmptr->param[1] = ' ';
+    }
+	x = str2int(str,&n);
+	fmptr->param[0] = x;
+	str += n;
+
 	if((ch = isformdir(*str))) {
 		fmptr->mode = ch;
 		return(str + 1);
