@@ -2,7 +2,7 @@
 /* file scanner.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996-2002 O.Forster
+Copyright (C) 1996-2004 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -34,14 +34,15 @@ Email   forster@mathematik.uni-muenchen.de
 ** scanning input
 **
 ** date of last change
-** 95-03-21
-** 97-01-24     moved skiptobel() to file.c
-** 97-04-11     changed trimblanks()
-** 97-07-04     changed readln to handle multi-line integers
-** 98-04-05     changed readln to handle reading empty strings
-** 98-10-06     readln: corrected handling of EOF
-** 98-11-14     nexttok: handle DOS line endings under UNiX
-** 99-06-15     readln: #ifdef Win32GUI, changed peekchar
+** 1995-03-21
+** 1997-01-24     moved skiptobel() to file.c
+** 1997-04-11     changed trimblanks()
+** 1997-07-04     changed readln to handle multi-line integers
+** 1998-04-05     changed readln to handle reading empty strings
+** 1998-10-06     readln: corrected handling of EOF
+** 1998-11-14     nexttok: handle DOS line endings under UNiX
+** 1999-06-15     readln: #ifdef Win32GUI, changed peekchar
+** 2004-02-09     numval: corrected value of count
 */
 
 #include "common.h"
@@ -982,7 +983,7 @@ int maxlen;
         *str = nextch();
         return(intinp(nextch,str,maxlen));
     }
-#ifdef GF2NINTEGER
+/* #ifdef GF2NINTEGER */
     else if(ch == '2' && isbasemark(ch1,2)) {
         nextch();
         *str = nextch();
@@ -991,7 +992,7 @@ int maxlen;
             tok = GF2NTOK;
         return(tok);
     }
-#endif /* GF2NINTEGER */
+/* #endif  GF2NINTEGER */
     else if(!isdecdigit(ch))
         return(aERROR);
     curbase = 10;
@@ -1253,8 +1254,7 @@ int *pcount;
         count = 0;
     else {
         count = cStrptr - buf;
-        if(*cStrptr)
-            count--;
+        count--;
     }
     *pcount = count;
 
