@@ -38,6 +38,7 @@ Email	forster@mathematik.uni-muenchen.de
 ** 1997-04-13: newreflsym(), neworg (newintsym)
 ** 2000-12-28: removed some debugging code
 ** 2002-04-19: mkgf2n, mk0gf2n
+** 2004-11-29: changed mkcopy, mkcopy0
 */
 
 #include "common.h"
@@ -57,6 +58,7 @@ PUBLIC truc newsym	_((char *name, int flg, truc bind));
 PUBLIC truc newsymsig	_((char *name, int flg, wtruc bind, int sig));
 PUBLIC truc new0symsig	_((char *name, int flg, wtruc bind, int sig));
 PUBLIC truc mkcopy	_((truc *x));
+PUBLIC truc mkcopy0  _((truc *x));
 PUBLIC truc mkarrcopy	_((truc *x));
 PUBLIC truc mkinum	_((long n));
 PUBLIC truc mkarr2	_((unsigned w0, unsigned w1));
@@ -102,9 +104,18 @@ PRIVATE void streamaux	_((FILE *file, int mode, struct stream *ptr));
 
 PRIVATE truc scratchsym;
 
+/* #define STORLOG */
+#ifdef STORLOG
+FILE *logf;
+static char *Logfile = "storlog.txt";
+static char logbuf[82];
+#endif
 /*----------------------------------------------------------------*/
 PUBLIC void inistore()
 {
+#ifdef STORLOG
+logf = fopen(Logfile,"w");
+#endif
 	scratchsym = newselfsym("",sINTERNAL);
 }
 /*----------------------------------------------------------------*/
@@ -409,7 +420,7 @@ int sig;
 ** Stellt Kopie eines Objekts *x her (top-level)
 ** Es wird vorausgesetzt, dass x bei gc geschuetzt ist
 */
-PUBLIC truc mkcopy(x)
+PUBLIC truc mkcopy0(x)
 truc *x;
 {
 	truc *ptr;
@@ -426,6 +437,29 @@ truc *x;
 }
 /*--------------------------------------------------------*/
 /*
+** Stellt Kopie eines Objekts *x her (top-level)
+** Es wird vorausgesetzt, dass x bei gc geschuetzt ist
+*/
+PUBLIC truc mkcopy(x)
+truc *x;
+{
+	truc *ptr;
+	truc obj;
+	unsigned int len;
+	int flg = *FLAGPTR(x);
+
+	if((flg & FIXMASK))
+		return(*x);
+	len = obj4size(flg,TAddress(x));
+#ifdef STORLOG
+fprintf(logf,"mknumcopy: flg = %d; obj4size = %d\n",flg,len);
+#endif
+	obj = newobj(flg,len,&ptr);
+	cpy4arr(TAddress(x),len,ptr);
+	return(obj);
+}
+/*--------------------------------------------------------*/
+/*
 ** Stellt Kopie eines Objekts *x her (Array beliebigen Ranges oder record)
 ** Es wird vorausgesetzt, dass x bei gc geschuetzt ist
 */
