@@ -2,7 +2,7 @@
 /* file terminal.c
 
 ARIBAS interpreter for Arithmetic
-Copyright (C) 1996 O.Forster
+Copyright (C) 1996-2002 O.Forster
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ Address of the author
 	Theresienstr. 39
 	D-80333 Muenchen, Germany
 
-Email	forster@rz.mathematik.uni-muenchen.de
+Email	forster@mathematik.uni-muenchen.de
 */
 /****************************************************************/
 /*
@@ -33,14 +33,15 @@ Email	forster@rz.mathematik.uni-muenchen.de
 ** terminal input
 **
 ** date of last change
-** 95-01-25	str0[] in initerm
-** 95-03-10	fixed bug in Fsaveinput
-** 95-03-22	treadline changed
-** 97-03-31	moved comprline inside #ifdef PAGEINPUT
-** 97-04-13	reorg (newintsym)
-** 98-10-28     adjustments for Win32GUI
-** 99-04-27	HISTBOX
-** 99-05-11     fixed problems with signed char in expandtabs
+** 1995-01-25	str0[] in initerm
+** 1995-03-10	fixed bug in Fsaveinput
+** 1995-03-22	treadline changed
+** 1997-03-31	moved comprline inside #ifdef PAGEINPUT
+** 1997-04-13	reorg (newintsym)
+** 1998-10-28   adjustments for Win32GUI
+** 1999-04-27	HISTBOX
+** 1999-05-11   fixed problems with signed char in expandtabs
+** 2002-02-16   testcomment
 */
 
 #include "common.h"
@@ -54,7 +55,7 @@ Email	forster@rz.mathematik.uni-muenchen.de
 
 #include "logscr.inc"
 
-#ifdef Win32GUI
+#ifdef genWinGUI
 #define HISTBOX
 #endif
 
@@ -65,17 +66,20 @@ PUBLIC void dumpinput	_((void));
 PUBLIC char *treadline	_((void));
 PUBLIC void historyout	_((int flg));
 
-PUBLIC truc historsym, savinsym;
+PUBLIC truc historsym, savinsym, bufovflsym;
 
+#ifdef genWinGUI
+PUBLIC int testcomment(char *buf);
+#endif
 /*--------------------------------------------------------*/
 PRIVATE void loadinput	_((char *str));
-PRIVATE int tinput	_((void));
+PRIVATE int tinput		_((void));
 PRIVATE int expandtabs	_((char *dest, char *src, int tabwidth));
 PRIVATE truc Fsaveinput	 _((int argn));
-PRIVATE int filout	_((int ch));
+PRIVATE int filout		_((int ch));
 PRIVATE int getinpbuf	_((int n));
-PRIVATE void inploop	_((void));
-PRIVATE int endtest	_((int curline));
+PRIVATE int inploop		_((void));
+PRIVATE int endtest		_((int curline));
 PRIVATE void protocinput  _((byte *str));
 PRIVATE void inputout	_((byte *str, ifun putfun));
 PRIVATE void historydisp  _((void));
@@ -93,13 +97,13 @@ PRIVATE truc getkeysym;
 #ifdef PAGEINPUT
 PRIVATE truc Floadedit	_((void));
 PRIVATE void getloadedit  _((void));
-PRIVATE int liesein	_((char *buffer, FILE *fptr));
+PRIVATE int liesein		_((char *buffer, FILE *fptr));
 PRIVATE char *comprline	 _((char *cpt, char *buf));
 PRIVATE int processkey	_((int key));
 PRIVATE int printable	_((int ch));
 PRIVATE void repaint	_((int startrow, int line0, int n));
 PRIVATE void curdown	_((int curline));
-PRIVATE void curup	_((int curline));
+PRIVATE void curup		_((int curline));
 PRIVATE void clrzeilrest  _((int curline, int col));
 PRIVATE void delzeile	_((int curline));
 PRIVATE void backspace	_((int curline));
@@ -109,7 +113,7 @@ PRIVATE void tableft	_((int curline, int tabwidth));
 PRIVATE void startpage	_((void));
 PRIVATE void endpage	_((void));
 PRIVATE void retbreak	_((int curline));
-PRIVATE void opennl	_((void));
+PRIVATE void opennl		_((void));
 PRIVATE void crnewline	_((int curline));
 
 PRIVATE truc loadedsym;
@@ -168,34 +172,40 @@ PRIVATE struct {
 /*------------------------------------------------------------*/
 PUBLIC void initerm()
 {
-        static trucptr ptrarr[4+HISTIMAX];
-        static char inam[] = "$_";
+	static char text[24];
+	static trucptr ptrarr[4+HISTIMAX];
+	static char inam[] = "$_";
 	static char str0[] = " 0.";
 	truc tempsym;
-        int k;
+	int k;
 
-        inpPtr = ptrarr + 3;
+	inpPtr = ptrarr + 3;
 	str0[0] = strlen(prompt) + 1;
 	nullinp = mkstr(str0);
 
-        for(k=-3; k<=HISTIMAX; k++) {
+	for(k=-3; k<=HISTIMAX; k++) {
             inam[1] = 'd'+k;
             tempsym = newsym(inam, sINTERNVAR, nullinp);
             inpPtr[k] = SYMBINDPTR(&tempsym);
-        }
+	}
 #ifdef HISTBOX
-        HB_ini();
+	HB_ini();
 #endif
-	historsym = newintsym("!",sINTERNAL, constone);
-	savinsym  = newsymsig("save_input",sFBINARY,(truc)Fsaveinput,s_12ii);
+	historsym = newintsym("!",sINTERNAL, (wtruc)0);
+    SYMbind(historsym) = constone;
+#ifndef genWinGUI
+	savinsym  = newsymsig("save_input",sFBINARY,(wtruc)Fsaveinput,s_12ii);
+#endif /* genWinGUI */
 
 #ifdef PAGEINPUT
-	loadedsym = newsymsig("load_edit", sFBINARY,(truc)Floadedit, s_bs);
+	loadedsym = newsymsig("load_edit", sFBINARY,(wtruc)Floadedit, s_bs);
 #endif
 
 #ifdef GETKEY
-	getkeysym = newsymsig("get_key",   sFBINARY,(truc)Fgetkey, s_0);
+	getkeysym = newsymsig("get_key",   sFBINARY,(wtruc)Fgetkey, s_0);
 #endif
+    s1form(text,"more than ~D lines",(wtruc)BUFLINES);
+	bufovflsym = newselfsym(text,sINTERNAL);
 }
 /*------------------------------------------------------------*/
 #ifdef HISTBOX
@@ -319,24 +329,31 @@ PUBLIC char *treadline()
 {
 	char *str;
 	int ch;
+	int ret = 1;
 
 	if(newinput) {
-#ifndef Win32GUI
+#ifndef genWinGUI
 		if(!Hist_out)
 			fnewline(tstdout);
 #endif
 		STRMlineno(tstdin) = 0;
-		inploop();
+		ret = inploop();
 		getinpbuf(1);
 		inpcursor = 0;
 		newinput = 0;
 		endinput = 0;
 	}
 	str = TinpBuf;
-	ch = tinput();
-	while(ch >= ' ') {
-		*str++ = ch;
+	if(ret >= 0) {
 		ch = tinput();
+		while(ch >= ' ') {
+			*str++ = ch;
+			ch = tinput();
+		}
+	}
+	else {
+		*str++ = ZESC;
+		*str++ = '\001';
 	}
 	*str = EOL;
 	return(TinpBuf);
@@ -347,7 +364,7 @@ char *str;
 {
 	if(!Hist_out) {
 #ifdef HISTBOX
-	        HB_export(4);
+		HB_export(4);
 #endif
 		*inpPtr[4] = *inpPtr[3];
 		*inpPtr[3] = *inpPtr[2];
@@ -364,17 +381,18 @@ PRIVATE int tinput()
 	int ch;
 
 	ch = Input[inpcursor++];
-	if(ch >= ' ')
+	if(ch >= ' ') {
 		return(ch);
-	else if(ch != 0) {
+	}
+    else if(ch == 0) {
+		endinput = 1;
+		return(FORMFEED);
+	}
+	else {
 		if(ch == TABESC)
 			inpcursor++;
 		return(EOL);
 	}
-	else {
-		endinput = 1;
-		return(FORMFEED);
-	}
 }
 /*-------------------------------------------------------------------*/
 /*
@@ -423,6 +441,7 @@ int tabwidth;
 	return(len);
 }
 /*-------------------------------------------------------------------*/
+#ifndef genWinGUI
 PRIVATE truc Fsaveinput(argn)
 int argn;
 {
@@ -489,6 +508,7 @@ int argn;
   errexit:
 	return(false);
 }
+#endif /* genWinGUI */
 /*-------------------------------------------------------------------*/
 PRIVATE int filout(ch)
 int ch;
@@ -499,15 +519,15 @@ int ch;
 PRIVATE int getinpbuf(n)
 int n;
 {
-        int ret = n;
+	int ret = n;
 #ifdef HISTBOX
-        if(n > HISTIMAX) {
-            ret = HB_import(n);
-            n = 0;
-        }
+	if(n > HISTIMAX) {
+		ret = HB_import(n);
+		n = 0;
+	}
 #endif
 	Input = (byte *)STRINGPTR(inpPtr[n]);
-        return(ret);
+	return(ret);
 }
 /*-------------------------------------------------------------------*/
 PRIVATE int endtest(curline)
@@ -516,6 +536,7 @@ int curline;
 	char *str;
 	int ch;
 	int k, k1;
+	int ret = 0;
 
 	k = L_trimlen(curline);
 
@@ -524,17 +545,21 @@ int curline;
 	if(Col <= k)
 		ch = 0;
 #endif
-	if(ch == '.' || ch == '?')
-		return(1);
-
-	if(curline == L_efffirst()) {
+	if(ch == '.' || ch == '?') {
+		ret = 1;
+	}
+	else if(curline == L_efffirst()) {
 		k1 = L_indent(curline);
 		str = L_linerest(curline,k1+1);
 		if(str[0] == '!' || strcmp(str,quitstring) == 0) {
-			return(1);
+			ret = 1;
 		}
 	}
-	return(0);
+	if(ret == 1) {
+		if(L_insidecomment(curline) == 1)
+			ret = 0;
+	}
+	return(ret);
 }
 /*-------------------------------------------------------------------*/
 PRIVATE void protocinput(str)
@@ -597,81 +622,106 @@ int k;
 	k = getinpbuf(k);
 	L_expand(Input);
 	display();
-        return k;
+	return k;
 }
 /*-------------------------------------------------------------------*/
-#ifdef Win32GUI
+#ifdef genWinGUI
 PUBLIC int cpyprevinput(buf,k)
 char *buf;
 int k;
 {
 	char InpBuf[BUFLINES][LINELEN];
-        int i, n;
-        char *str;
+	int i, n;
+	char *str;
 
-        Col0 = L_iniscr(InpBuf,prompt);
+	Col0 = L_iniscr(InpBuf,prompt);
 	k = getinpbuf(k);
 	L_expand(Input);
 	n = L_efflen() - 1;
 	for(i=0; i<=n; i++) {
 		str = L_line(i) + (i==0 ? Col0-1 : 0);
-                buf += strcopy(buf,str);
-                if(i < n)
-                        buf += strcopy(buf,"\r\n");
+		buf += strcopy(buf,str);
+		if(i < n) {
+#ifdef Win32GUI
+			buf += strcopy(buf,"\r\n");
+#else
+			buf += strcopy(buf,"\n");
+#endif
+		}
 	}
-        *buf = '\0';
-        return k;
+	*buf = '\0';
+	return k;
 }
+/*-------------------------------------------------------------------*/
+PUBLIC int testcomment(buf)
+char *buf;
+{
+	char InpBuf[BUFLINES][LINELEN];
+	int curline;
+	int ret;
+
+	Col0 = L_iniscr(InpBuf,prompt);
+	curline = L_text2blatt(buf)-1;
+	if(curline < 0)
+		ret = -1;
+	else
+		ret = L_insidecomment(curline);
+	return ret;
+}
+/*-------------------------------------------------------------------*/
 #endif
 /*-------------------------------------------------------------------*/
 #ifndef PAGEINPUT
 /*-------------------------------------------------------------------*/
-PRIVATE void inploop()
+PRIVATE int inploop()
 {
 	char InpBuf[BUFLINES][LINELEN];
 	char linebuf[LINELEN];
-	int weiter = 1;
+	int ret = 1;
 	char *str;
-#ifdef Win32GUI
-        char *winbuf;
-        int k, ch;
+#ifdef genWinGUI
+	char *winbuf;
+	int k, ch;
 #endif
 
 	Col0 = L_iniscr(InpBuf,prompt);
-#ifndef Win32GUI
+#ifndef genWinGUI
 	if(Hist_out) {
 		historydisp();
 	}
 	else {
 		fprintstr(tstdout,prompt);
-		while(weiter) {
+		while(1) {
 			str = fgets(linebuf,LINELEN-3,stdin);
 			if(str == NULL) {
 			    strcopy(linebuf,quitstring);
 			}
-			weiter = processline(linebuf);
+			ret = processline(linebuf);
+			if(ret <= 0)
+				break;
 		}
 	}
-#else   /* ifdef Win32GUI */
-        winbuf = getWinscrBuf();
-        while(weiter) {
-                str = linebuf;
-                for(ch=*winbuf++,k=1; (ch && ch!='\n' && k<=LINELEN-3); k++) {
-                        *str++ = ch;
-                        ch = *winbuf++;
-                }
-                *str = '\0';
-                weiter = processline(linebuf);
-                if(ch == 0)
-                        weiter = 0;
-        }
-#endif  /* ?Win32GUI */
+#else   /* ifdef genWinGUI */
+	winbuf = getWinscrBuf();
+	while(1) {
+		str = linebuf;
+		for(ch=*winbuf++,k=1; (ch && ch!='\n' && k<=LINELEN-3); k++) {
+			*str++ = ch;
+			ch = *winbuf++;
+		}
+		*str = '\0';
+		ret = processline(linebuf);
+        if(ret <= 0 || ch == 0)
+			break;
+	}
+#endif  /* ?genWinGUI */
 	L_compress();
 	str = InpBuf[0];
 	if(Log_on) {
 		protocinput((byte *)str);
-        }
+	}
 	loadinput(str);
+	return(ret);
 }
 /*-------------------------------------------------------------------*/
 PRIVATE int processline(line)
@@ -679,17 +729,20 @@ char *line;
 {
 	char expandbuf[EXPANDMAX+1];
 	int lineno, len, n;
-	int ret;
+	int ret, fertig;
 
 	lineno = L_pagelen() - 1;
 	len = L_len(lineno);
 	n = expandtabs(expandbuf,line,ETABWIDTH);
 	L_strappend(lineno,len+1,expandbuf);
 	ret = L_insert(lineno+1);
-	if(ret == 0)
-		return(ret);
+    fertig = endtest(lineno);
+	if(ret == 0 && fertig == 0)
+		return(-1);
+	else if(fertig)
+		return(0);
 	else
-		return(!endtest(lineno));
+		return(ret);
 }
 /*-------------------------------------------------------------------*/
 PRIVATE void display()
@@ -706,7 +759,7 @@ PRIVATE void display()
 /*-------------------------------------------------------------------*/
 #else /* ifdef PAGEINPUT */
 /*-------------------------------------------------------------------*/
-PRIVATE void inploop()
+PRIVATE int inploop()
 {
 	char InpBuf[BUFLINES][LINELEN];
 	int weiter = 1;
@@ -736,6 +789,7 @@ PRIVATE void inploop()
 	if(Log_on)
 		protocinput((byte *)str);
 	loadinput(str);
+	return(1);
 }
 /*-------------------------------------------------------------------*/
 PRIVATE void display()
